
// ****************************************************************************
//
// Routine:      acReadGlobal
// Author:       T.Oest
//
// Description:  read global alignment constants from database
//                
//
// Modification:
// transformation routine for GEDE table called                     16/09/99 TO
// access by key or data                                            01/12/99 TO
// include return code                                              07/02/00 TO
// GEDE transformation moved to acread.C                            06/03/00 TO
// corrected include ptah bug                                       16/08/00 AG
//
// ****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>
#include <limits.h>

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// ONLINE includes
#include "table.h"
#include "esp.h"

// prototypes
int  acReadGlobalByTime(int, char *);
int  acReadGlobalByKey(int, char *);
void ackuexec(char * KuipCommand);
 
// globals
static int first = 0;
int acReadGlobalSwitch = 1;

//*****************************************************************
// function
//*****************************************************************

int
acReadGlobal( int KTstatus )
{
  ARTE_mark( "acReadGlobal", 0 );

 int flag = 0;
 int ACGAupdate = 0;

 int KeyStatus = KTstatus;
 if (KTstatus == 0) KeyStatus = 2;

//==================================================================
// no action if switched off
//==================================================================

 if (acReadGlobalSwitch != 1){

   cout << " acReadGlobal switched off: no global alignment constants read in " << endl; 

   return -1;

 };


//==================================================================
// no action for MC
//==================================================================

  if (ArteTable<MTRA>::size() > 0 ) return 0;

//==================================================================
// database server (default: /SETUP_ALIGN)
//                  can be changed by setting DB_GLOBAL_ALIGNMENT
//==================================================================

 char defaultserver[13];
 sprintf(defaultserver,"/SETUP_ALIGN");
 char * dbserver;

 if (getenv("DB_GLOBAL_ALIGNMENT") == NULL){
   dbserver = defaultserver;
 } else {
   dbserver = getenv("DB_GLOBAL_ALIGNMENT");
 };


//==================================================================
// get LUN number for database
//==================================================================
  char LunName[5] = {'D','B','G','A',0};
  char * KuipCommand =  (char*) malloc(max(128,
                                       int(24+strlen(dbserver))));

  if (first == 0){
    first = 1;
    sprintf (KuipCommand,"IOFILE %s %s",LunName,dbserver);
    ackuexec(KuipCommand);
  };
 

//==================================================================
// acces by key table
//==================================================================

 if (KeyStatus >= 1) ACGAupdate = acReadGlobalByKey(KeyStatus, dbserver);

 flag = ACGAupdate;

//==================================================================
// acces by date if KeySstatus < 1 or access by key not sucessfull
//==================================================================

 if (KeyStatus < 1 || ACGAupdate < 0) {

    ACGAupdate = acReadGlobalByTime(KeyStatus, dbserver);
    if (ACGAupdate <  0) flag = -4;
    if (ACGAupdate == 1) flag = 2;
 };

//==================================================================
// check if ACGA table exist
//==================================================================

 if (ArteTable<ACGA>::size() == 0) return -3;


 return flag;
}
