//                              -*- Mode: C++ -*-
//: EAClusterAnalyzer.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 4/4/97
// Last Update ........
//


#include "care/EAClusterAnalyzer.hh"
#include "care/EMClusterTree.hh"
#include "care/EMTrackOnEcal.hh"
#include "care/EMCellFraction.hh"

/* 
CLASS
  EAClusterAnalyzer
  Abstract base class for cluster analysis

*/

//////////////////// EAClusterAnalyzer Class Definitions ////////////////

// No functions

//////////////////// EAClusterAnalyzer Class Definitions ////////////////
    
EAClusterAnalyzer::EAClusterAnalyzer(){}
EAClusterAnalyzer::~EAClusterAnalyzer(){}
  

//--
// Cluster Analyzer
void EAClusterAnalyzer::doAnalysis(const EMCluster * clus){
  // perform operation on all elements of the cluster
  for(const EMClusterMember* child = clus->begin(); 
      child!=clus->end();  child = child->next())
      child->analyzeMe(*this);
}

//--
// Cluster Tree Analyzer
void EAClusterAnalyzer::doAnalysis(const EMClusterTree *  clus){
  // perform operation on all elements of the cluster
  for(const EMClusterMember* child = clus->begin(); 
      child!=clus->end();  child = child->next())
      child->analyzeMe(*this);
}

//--
// TrackOnEcal
void EAClusterAnalyzer::doAnalysis(const EMTrackOnEcal *  track){
  // perform operation as it where a general ClusterMember
  const EMClusterMember* member = track;
  doAnalysis( member );
}


//--
// EMCellFraction
void EAClusterAnalyzer::doAnalysis(const EMCellFraction * cell){
  // perform operation as it where a general EMHitCell
  const EMHitCell * hitcell = cell;
  doAnalysis( hitcell );
}




