//                              -*- Mode: C++ -*-
//: dgcal.C
//
// Author: Mauro Villa   
// ECAL group
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 21/8/97
// Last Update ........
//
//
// The dgcal routine provides the digitization of the calorimeter signals
// stored in the MIMP bank.
// Three different kind of digitization can be choosen by external kuip
// command that correspond to three different levels of accuracy in
// the reproduction of the real detector:
//
//    Level  Input  Output  Description
//      0    MCAL   HITC    Standar digitalization for arte-01-05 and earlier
//                          arte versions. The Q entry in the HITC bank
//                          contain an uncalibrated energy responce (in GeV)
//      1    MCAL   HITC    The Q entry in the HITC bank contains a
//                          integer number (in floating representation) that 
//                          is the "likely" responce of a readout ADC channel 
//                          to the (calibrated) signal observed.
//      2    MCAL   DCAL    The digitization output is exactly the Second Level
//                          buffer of the calorimeter. The HITC bank will be
//                          created and filled in the reconstruction routines.


#include "care/dgcal.hh"
#include "arte/ArteMessage.hh"
#include "care/CmdCareDglev.hh"
#include "care/care.hh"
#include "care/caredata.hh"


void dgcal_(int *isl){

#ifndef ONLINE
  if( ArtePointer<ECBO>::getLink()()==0){
#else
  if( ArtePointer<ECBO>::nrow()==0){
#endif 
    if( !ECalInfo.init) careinit_();
    dgcalinit_();
  }

  int lev = CmdCareDglev::dglevel();  // get the ECAL digi level 

  switch(lev){
  case 0:
    // Standar digitalization for arte-01-05  MIMP -> HITC (Q=energy)
    dgcal0_(isl);
    break;
  case 1:
    // Digitalization for arte >01-05  MIMP -> HITC (Q= ADC channel)
    dgcal1_(isl);
    break;
  case 2:
    // Digitalization till the Second Level Buffer  MIMP -> DCAL
    dgcal2_(isl);
    break;
  default:
    // Error: at least one digitization level must be choosen
    arteMsg<<" DGCAL: wrong digitization level !!";
    arteMsg.error();
    dgcal0_(isl);
  }
}
