
// *****************************************************************************
//
// source:    DQ2DB.C
//
// type:      source code
//
// created:   01 Aug 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    DQ2DBS
//
// called by: DQ_stop
//
// purpose:   write DQ info to DBS
//
// hints:
//
// changes:
// 01 Aug 2000 (AG) creation
// 07 Aug 2000 (AG) introduced Kumac2DBS
// 10 Aug 2000 (AG) corrected kumac file name handling
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <string.h>

// ONLINE includes
#include "table.h"
#include "rhp.h"
#include "gth.h"

// ARTE includes
#include "arte/ArteFlag.h"
#include "farm/farm.hh"

// prototypes
void WriteTstDetails( char *dbs, int major, int minor );
void Kumac2DBS( char *kumac, char *DBS, int major, int minor );
//
// function code
//
void
DQ2DBS()
{
  ARTE_mark( "DQ2DBS 0", 0 );

  // databases
  char *histDBS = getenv( "DQHISTDBS" );
  if( !histDBS ){
    cout << "DQ2DBS: ERROR histograms  DBS: " << histDBS << endl;
    return;
  }
  else {
    cout << "DQ2DBS: histograms  DBS: " << histDBS << endl;
  }

  char *infoDBS = getenv( "DQINFODBS" );
  if( !infoDBS ){
    cout << "DQ2DBS: ERROR information DBS: " << infoDBS << endl;
    return;
  }
  else {
    cout << "DQ2DBS: information DBS: " << infoDBS << endl;
  }

  // stamp
  int major       = time( NULL );
  int minor       = int( 10000 * arteflag_.artevers );
  int Ckey        = 1;
  int Ckeyvers    = 1;
  int dstfile     = 1;
  gth_Stamp stamp = pack_stamp( Ckey, Ckeyvers, major,
				arteflag_.numexp,
				arteflag_.numrun,
				dstfile,
				arteflag_.nevents,
				6 );

  // info DBS
  write_stamp_to_dbs( infoDBS, stamp );

  // histo DBS
  dqs_write_hists_to_dbs( histDBS, major, minor );

  // details
  WriteTstDetails( infoDBS, major, minor );
  
  // dump kumac
  char kumac[80];
  memset( kumac, '\0', 80 );
  //cout << "arteflag_.progname:" << arteflag_.progname << endl;
  char *p = strtok( arteflag_.progname, " " );
  strcpy( kumac, p );
  //cout << "kumac:" << kumac << endl;
  strcat( kumac, ".kumac" );
  cout << "DQ2DBS: kumac: " << kumac << endl;

  Kumac2DBS( kumac, infoDBS, major, minor );

  ARTE_mark( "DQ2DBS 1", 0 );
  return;
}

//
// function code
//
void
WriteTstDetails( char *infoDBS, int major, int minor )
{
  ARTE_mark( "WriteTstDetails 0", 0 );
  
  char **details;
  
  int NLINES = 2;
  details = (char **) malloc( NLINES * sizeof( char ** ) ); 
  for( int i=0; i<NLINES; i++) details[i] = (char *) malloc( 300 );
  
  sprintf( details[0], "Arte Version: %6f",      arteflag_.artevers );
  sprintf( details[1], "IP machine address: %u", arteflag_.ipaddr );
  //  sprintf(details[2],"Hostname %s",arteflag_.hostname);
  //  sprintf(details[3],"Bintype %s",arteflag_.bintype);
  
  dqs_write_tst_details_to_dbs( infoDBS, major, minor, NLINES, details );

  ARTE_mark( "WriteTstDetails 1", 0 );
}

//
// write kumac to DBS
//
void
Kumac2DBS( char *kumac,
	   char *infoDBS, 
	   int major, 
	   int minor )
{
  ARTE_mark( "Kumac2DBS 0", 0 );
  
  // check existence of file
  FILE *f = fopen( kumac, "r" );
  if( !f ){
    cout << "Kumac2DBS: ERROR: file:" << kumac << " NOT existing." << endl;
    return;
  }
  
  // open DB
  Tables x = OpenTables( infoDBS, 0 );
  if( x ){
    cout << "Kumac2DBS: OpenTables DBS:" << infoDBS << " opened." << endl;
  }
  else {
    cout << "Kumac2DBS: ERROR: OpenTables DBS:" << infoDBS << " NOT opened." << endl;
    return;
  }

  // put table
  int rc = PutTableFile( x, kumac, "KumacTst", major, minor, 0 );
  if( !rc ){
    cout << "Kumac2DBS: PutTableFile" << endl;
  }
  else {
    cout << "Kumac2DBS: ERROR: PutTableFile rc=" << rc << endl;
    return;
  }

  // close DB
  CloseTables( x );
  
  ARTE_mark( "Kumac2DBS 1", 0 );
}
