
// *****************************************************************************
//
// source:    DQ_FARM_rec_evnt.C
//
// type:      source code
//
// created:   17 Nov 1999
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ_event()
//
// changes:
// 17 Nov 1999 (AG) creation
// 13 Mar 2000 (AG) monitor RTRA, RSEG, RCCL, RVER
// 08 May 2000 (AG) event classification flag
// 12 May 2000 (AG) histogram 0 if no class bit set
// 18 May 2000 (AG) timings of reco modules
// 22 May 2000 (AG) comment 2-d histograms; polish printout
// 24 May 2000 (AG) histogram bytes
// 26 May 2000 (AG) histogram time_rcevnt here
// 31 May 2000 (AG) change scale for Rnnn
//  5 Jun 2000 (AG) add timing routine
//  6 Jun 2000 (AG) determine OTR and RICH reco rejection
//  7 Jun 2000 (AG) use new timing table EVTI
//  8 Jun 2000 (AG) some polishing of printout
// 16 Jun 2000 (AG) improve mean calculation
// 19 Jun 2000 (AG) calculate average number off hits in OTR
// 21 Jun 2000 (AG) printout every 200 events
// 23 Jun 2000 (AG) add time vs hits histos
// 26 Jun 2000 (AG) added ARTE_mark()
// 27 Jun 2000 (AG) change RICH cuts
// 01 Aug 2000 (AG) get rid of old stuff
// 03 Aug 2000 (AG) disabled DQ_rej and introduced DQ_class 
// 04 Aug 2000 (AG) polished printout
//
// ******************************************************************

// standard includes
#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>

// ONLINE includes
#include "rhp.h"
#include "grr.h"

// Arte includes
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_FARM.h"

// prototypes
void DQ_FARM_rec_timing();
void DQ_FARM_rec_rejection();
void DQ_FARM_rec_class();

// externals
extern char *LOGFILE;  // logfile
extern int  FATIME;    // flag for printout of timing
extern int  FADEB;     // debug flag
extern int  *classSum; // classification bits

//
// function code
//
int
DQ_FARM_rec_evnt()
{
  // show up
  ARTE_mark( "DQ_FARM_rec_evnt 0", 0 );

  // DST flag
  ArtePointer<EVHD> evhd( 1 );
  int flagdst = evhd->flagdst;

  // l4t event classification
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 1 ), 0, 1 );
  for( int bit=0; bit<32; bit++ ){
    bool j = ( flagdst >> bit ) & 1;
    if( j ) rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 1 ), bit + 1, 1 );
  }

  // reco tables
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 10 ), ArteTable<RTRA>::size(), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 11 ), ArteTable<RSEG>::size(), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 12 ), ArteTable<RCCL>::size(), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 13 ), ArteTable<RVER>::size(), 1 );

  // EVRC
  if( ArteTable<EVRC>::size() == 0 ) return 0;
  ArtePointer<EVRC> evrc = *( ArteTable<EVRC>::end() - 1 );
  
  // EVTI
  if( ArteTable<EVTI>::size() == 0 ) return 0;
  ArtePointer<EVTI> evti = *( ArteTable<EVTI>::end() - 1 );

  // reco standalone modules
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 100 ), evrc->cputime, 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 101 ), evti->vds,     1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 104 ), evti->ranger,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 105 ), evti->rich,    1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 106 ), evti->trd,     1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 107 ), evti->ecal,    1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 108 ), evti->muon,    1 );

  // reco modules
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 112 ), evti->marple, 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 114 ), evti->refit,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 116 ), evti->pid,    1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 117 ), evti->vertex, 1 );

  // hits per detector
  int hitsVDS  = 0;
  int hitsOTR  = 0;
  int hitsRICH = 0;
  for( ArteTableIterator<HITB> hitb; hitb.valid(); hitb.next() ){ 
    ArtePointer<GEDE> gede( hitb->gede ); 
    if( gede->cmp == 1 ) hitsVDS++;
    if( gede->cmp == 2 ) hitsOTR++;
    if( gede->cmp == 5 ) hitsRICH++;
  }

  // correlations
  rhp_hfill( DQ_ID( COMP_FARM, LVL_REC, 201 ), hitsVDS,  evti->vds,    1 );
  rhp_hfill( DQ_ID( COMP_FARM, LVL_REC, 204 ), hitsOTR,  evti->ranger, 1 );
  rhp_hfill( DQ_ID( COMP_FARM, LVL_REC, 205 ), hitsRICH, evti->rich,   1 );  

  // monitor timing
  ARTE_mark( "DQ_FARM_rec_timing",0 );
  DQ_FARM_rec_timing();

  // monitor rejection
  //ARTE_mark( "DQ_FARM_rec_rejection", 0 );
  //DQ_FARM_rec_rejection();

  ARTE_mark( "DQ_FARM_rec_evnt 1", 0 );
  
  return 0;
}

//
// statistics
//

// update mean
inline float mean( float mean, float x, int n ){
  return ( float( n - 1 ) / n ) * mean + fabsf( x ) / n;
}

// update variance 
//inline float vari( float vari, float mean, float x, int n ){
//  return ( float( n - 1) / n ) * vari + ( fabsf( x ) - mean ) * ( fabsf( x ) - mean ) / n;
//}

//
// function DQ_FARM_rec_timing()
//
void
DQ_FARM_rec_timing()
{
  ARTE_mark( "DQ_FARM_rec_timing 0", 0 );

  // EVRC
  if( ArteTable<EVRC>::size() == 0 ) return;
  ArtePointer<EVRC> evrc = *( ArteTable<EVRC>::end() - 1 );

  // EVTI
  if( ArteTable<EVTI>::size() == 0 ) return;
  ArtePointer<EVTI> evti = *( ArteTable<EVTI>::end() - 1 );

  // counter
  int n = arteflag_.nevents;
  
  static float mean_cputime = 0;
  static float mean_vds     = 0;
  static float mean_ranger  = 0;
  static float mean_rich    = 0;
  static float mean_ecal    = 0;
  static float mean_muon    = 0;
  static float mean_marple  = 0;
  static float mean_refit   = 0;
  static float mean_pid     = 0;
  static float mean_vertex  = 0;

  // mean
  mean_cputime = mean( mean_cputime, evrc->cputime, n );
  mean_vds     = mean( mean_vds,     evti->vds,     n );
  mean_ranger  = mean( mean_ranger,  evti->ranger,  n );
  mean_rich    = mean( mean_rich,    evti->rich,    n );
  mean_ecal    = mean( mean_ecal,    evti->ecal,    n );
  mean_muon    = mean( mean_muon,    evti->muon,    n );
  mean_marple  = mean( mean_marple,  evti->marple,  n );
  mean_refit   = mean( mean_refit,   evti->refit,   n );
  mean_pid     = mean( mean_pid,     evti->pid,     n );
  mean_vertex  = mean( mean_vertex,  evti->vertex,  n );
  
  // text
  static char time_txt[400];
  static char mean_txt[400];
  memset( time_txt, '\0', 400 );
  memset( mean_txt, '\0', 400 );
  sprintf( time_txt,
	   "TIME: [sec] RECO=%3.1f VDS=%4.2f Rang=%4.2f RICH=%4.2f ECAL=%4.2f MUON=%4.2f Marp=%4.2f Ref=%4.2f Pid=%4.2f Vtx=%4.2f",
	   evrc->cputime,
	   evti->vds,
	   evti->ranger,
	   evti->rich, 
	   evti->ecal, 
	   evti->muon, 
	   evti->marple,
	   evti->refit,
	   evti->pid,
	   evti->vertex
	   );
  sprintf( mean_txt,
	   "TIME: [sec] RECO=%3.1f VDS=%4.2f Rang=%4.2f RICH=%4.2f ECAL=%4.2f MUON=%4.2f Marp=%4.2f Ref=%4.2f Pid=%4.2f Vtx=%4.2f",
	   mean_cputime,
	   mean_vds,
	   mean_ranger,
	   mean_rich, 
	   mean_ecal, 
	   mean_muon, 
	   mean_marple,
	   mean_refit,
	   mean_pid,
	   mean_vertex
	   );

  // printout
  if( FATIME ){
    cout << " " << time_txt << endl;
    cout << " " << mean_txt << endl;
  }

  // erwin & log
  if( ( n == 50 ) || !( n%200 ) ){
    ARTE_error( GRR_SINFO, mean_txt, 0 );
    ARTE_log( LOGFILE, mean_txt );
  }
  
  ARTE_mark( "DQ_FARM_rec_timing 1", 0 );
}

//
// DQ_FARM_rec_rejection()
//
void
DQ_FARM_rec_rejection()
{
  ARTE_mark( "DQ_FARM_rec_rejection 0", 0 );

  // counter
  int n = arteflag_.nevents;

  //
  // OTR **********************************************************************
  //

  // OTR cutoff
  int  cutOTR   = 9999999;
  int  cutOTR10 = 10000;
  int  cutOTR12 = 12000;
  int  cutOTR15 = 15000;
  int  cutOTR20 = 20000;
  char *ccutOTR = getenv( "OtrOccupancyCut" );
  if( ccutOTR ) cutOTR = atoi( ccutOTR );
  
  // OTR hits
  int hitsOTR = 0;
  for( ArteTableIterator<HITB> hitb; hitb.valid(); hitb.next() ){ 
    ArtePointer<GEDE> gede( hitb->gede ); 
    if( gede->cmp == 2 ) hitsOTR++;
  }
  
  // average number of OTR hits
  static float mean_hitsOTR = 0;
  mean_hitsOTR = mean( mean_hitsOTR, float( hitsOTR ), n );

  // OTR count
  static int rejOTR   = 0;
  static int rejOTR10 = 0;
  static int rejOTR12 = 0;
  static int rejOTR15 = 0;
  static int rejOTR20 = 0;
  if( hitsOTR > cutOTR )   rejOTR++;
  if( hitsOTR > cutOTR10 ) rejOTR10++;
  if( hitsOTR > cutOTR12 ) rejOTR12++;
  if( hitsOTR > cutOTR15 ) rejOTR15++;
  if( hitsOTR > cutOTR20 ) rejOTR20++;

  // OTR print
  char otr_txt[200];
  memset( otr_txt, '\0', 200 );
  sprintf( otr_txt,
	   "REJECTION: [%%] mean=%6.0f cut=%6d rej=%3.1f 10000=%3.1f 12000=%3.1f 15000=%3.1f 20000=%3.1f",
	   mean_hitsOTR,
	   cutOTR,
	   float( rejOTR )   / n * 100,
	   float( rejOTR10 ) / n * 100,
	   float( rejOTR12 ) / n * 100,
	   float( rejOTR15 ) / n * 100,
	   float( rejOTR20 ) / n * 100 );

  //
  // RICH *********************************************************************
  //

  // RICH cutoff
  int  cutRICH   = 3000;
  int  cutRICH30 = 3000;
  int  cutRICH40 = 4000;
  int  cutRICH50 = 5000;
  int  cutRICH60 = 6000;

  // RICH hits
  int hitsRICH = 0;
  for( ArteTableIterator<HITB> hitb; hitb.valid(); hitb.next() ){ 
    ArtePointer<GEDE> gede( hitb->gede ); 
    if( gede->cmp == 5 ) hitsRICH++;
  }

  // average number of RICH hits
  static float mean_hitsRICH = 0;
  mean_hitsRICH = mean( mean_hitsRICH, float( hitsRICH ), n );

  // RICH count
  static int rejRICH   = 0;
  static int rejRICH30 = 0;
  static int rejRICH40 = 0;
  static int rejRICH50 = 0;
  static int rejRICH60 = 0;
  if( hitsRICH > cutRICH )   rejRICH++;
  if( hitsRICH > cutRICH30 ) rejRICH30++;
  if( hitsRICH > cutRICH40 ) rejRICH40++;
  if( hitsRICH > cutRICH50 ) rejRICH50++;
  if( hitsRICH > cutRICH60 ) rejRICH60++;

  // RICH print
  char rich_txt[200];
  memset( rich_txt, '\0', 200 );
  sprintf( rich_txt,
	   "REJECTION: [%%] mean=%6.0f cut=%6d rej=%3.1f 3000=%3.1f 4000=%3.1f 5000=%3.1f 6000=%3.1f",
	   mean_hitsRICH,
	   cutRICH,
	   float( rejRICH )   / n * 100,
	   float( rejRICH30 ) / n * 100,
	   float( rejRICH40 ) / n * 100,
	   float( rejRICH50 ) / n * 100,
	   float( rejRICH60 ) / n * 100 );

  //
  // printout
  //
  if( FADEB ){
    cout << " DQ_FARM_rec: OTR  hits=" << hitsOTR  << endl;
    cout << " DQ_FARM_rec: RICH hits=" << hitsRICH << endl;
    cout << " " << otr_txt  << endl;
    cout << " " << rich_txt << endl;
  }

  //
  // erwin & logfile
  //
  if( ( n == 50 ) || !( n%200 ) ){
    ARTE_error( GRR_SINFO, otr_txt,  0 );
    ARTE_error( GRR_SINFO, rich_txt, 0 );
    ARTE_log( LOGFILE, otr_txt );
    ARTE_log( LOGFILE, rich_txt );
  }

  ARTE_mark( "DQ_FARM_rec_rejection 1", 0 );
}

//
// DQ_FARM_rec_class()
//
void
DQ_FARM_rec_class()
{
  ARTE_mark( "DQ_FARM_rec_class 0", 0 );

  // counter
  int n = arteflag_.nevents;

  // classification statistics
  if( classSum ){
    static char str[300];
    static char txt[20];
    memset( str, '\0', 300 );
    memset( txt, '\0', 20 );
    strcpy( str, " CLASS: " );
    for( int i=0; i<16; i++ ){
      sprintf( txt, "#%d=%d", i+1, classSum[i] );
      strcat( str, txt );
    }

    // printout
    if( FADEB ) cout << str << endl;

    // erwin & log
    if( ( n == 50 ) || !( n%200 ) ){
      ARTE_error( GRR_SINFO, str, 0 );
      ARTE_log( LOGFILE, str );
    }
  }

  ARTE_mark( "DQ_FARM_rec_class 1", 0 );
}
