
// ********************************************************************
//
// Routine:      DQ_RECO_vertex
// Author:       T.Oest
//
// Description:  fill vertex DQ histograms
//
// Changes:
//  5 Jul 2000 (AG) called it DQ_RECO_vertex()
//  7 Jul 2000 (AG) polished printout
//
// ********************************************************************

// standrad includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

// ARTE includes
#include "arte/RSEG_cmp.hh"
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "rhp.h" 
#include "dq/DQ.h"

//
// function code
//
int
DQ_RECO_vertex_init()
{
  // show up
  ARTE_mark( "DQ_RECO_vertex_init 0", 0 );
  cout << "DQ_RECO_vertex_init" << endl;
  
  // histograms
  rhp_hbook2( DQ_ID( COMP_RECO,LVL_REC, 201 ), 
	      "RECO main vertices II station ", 100, -1, 1, 100, -1, 1 );
  rhp_hbook2( DQ_ID( COMP_RECO,LVL_REC, 202 ), 
	      "RECO main vertices I  station ", 100, -1, 1, 100, -1, 1 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 203 ), 
	      "RECO vertices per wire ", 8, 0.5, 8.5 );
  rhp_hbook2( DQ_ID( COMP_RECO,LVL_REC, 204 ), 
	      "RECO main vertices zx ", 100, -10, 5, 100, -1, 1 );
  rhp_hbook2( DQ_ID( COMP_RECO,LVL_REC, 205 ), 
	      "RECO main vertices zy ", 100, -10, 5, 100, -1, 1 );

  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 210 ), "RECO  x wire  Ia ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 211 ), "RECO  x wire  Ib ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 212 ), "RECO dx wire  Ii ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 213 ), "RECO dx wire  Io ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 214 ), "RECO  x wire IIa ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 215 ), "RECO  x wire IIb ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 216 ), "RECO dx wire IIi ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 217 ), "RECO dx wire IIo ", 100, -0.5, 0.5 );

  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 220 ), "RECO dy wire  Ia ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 221 ), "RECO dy wire  Ib ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 222 ), "RECO  y wire  Ii ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 223 ), "RECO  y wire  Io ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 224 ), "RECO dy wire IIa ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 225 ), "RECO dy wire IIb ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 226 ), "RECO  y wire IIi ", 100, -0.5, 0.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 227 ), "RECO  y wire IIo ", 100, -0.5, 0.5 );

  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 230 ), "RECO dz wire  Ia ", 100, -1, 1 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 231 ), "RECO dz wire  Ib ", 100, -1, 1 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 232 ), "RECO dz wire  Ii ", 100, -1, 1 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 233 ), "RECO dz wire  Io ", 100, -1, 1 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 234 ), "RECO dz wire IIa ", 100, -1, 1 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 235 ), "RECO dz wire IIb ", 100, -1, 1 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 236 ), "RECO dz wire IIi ", 100, -1, 1 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 237 ), "RECO dz wire IIo ", 100, -1, 1 );
 
  // show up
  ARTE_mark( "DQ_RECO_vertex_init 1", 0 );

  return 0;
}

//
// function code
//
int
DQ_RECO_vertex_evnt()
{
  // show up
  ARTE_mark( "DQ_RECO_vertex_evnt 0", 0 );

  //-----------------------------------------------------------------
  //  active wires
  //-----------------------------------------------------------------
  int wireFlag[8];
  int i = 0;
  
  for( ArteTable<GTAR>::iterator gtar = ArteTable<GTAR>::begin();
       gtar != ArteTable<GTAR>::end();
       ++gtar ) {
    if ( ((*gtar)->x * (*gtar)->x +(*gtar)->y * (*gtar)->y ) < 1){
      wireFlag[i] = 1;
    } else {
      wireFlag[i] = 0;
    }
    i++;
  }
  
  //-----------------------------------------------------------------
  //   main vertices
  //-----------------------------------------------------------------
  
  for( ArteTable<RVER>::iterator rver = ArteTable<RVER>::begin();
       rver != ArteTable<RVER>::end(); ++rver ) {
    
    if ((*rver)->btra == 0 ){
      
      //-----------------------------------------------------------------
      //   closest target wire
      //-----------------------------------------------------------------
      
      int igtar = 0;
      i = 0;
      float diff = 0.0;
      float dx, dy, dz;
      ArtePointer<GTAR> Vgtar;
      
      for( ArteTable<GTAR>::iterator gtar = ArteTable<GTAR>::begin();
	   gtar != ArteTable<GTAR>::end();
	   ++gtar ) {
	i++;
	if (wireFlag[i-1] ==1) {
	  
	  dx = ((*gtar)->x - (*rver)->xv)/(*gtar)->dx;
	  dy = ((*gtar)->y - (*rver)->yv)/(*gtar)->dy;
	  dz =  (*gtar)->z - (*rver)->zv;
	  
	  if ( fabs(dz) < 1. && 
	       (sqrt(dx*dx+dy*dy) < diff || igtar == 0)) {
	    
	    diff  = sqrt(dx*dx+dy*dy);
	    Vgtar = *gtar;
	    igtar = i;
	  }
	}
      }
      
      //-----------------------------------------------------------------
      //  
      //-----------------------------------------------------------------
      
      rhp_hfill(DQ_ID(COMP_RECO, LVL_REC, 204) ,(*rver)->zv, (*rver)->xv ,1);
      rhp_hfill(DQ_ID(COMP_RECO, LVL_REC, 205) ,(*rver)->zv, (*rver)->yv ,1);
      
      
      if ( igtar != 0){
	
	
	if ( Vgtar->z > -3.0 ){
          rhp_hfill(DQ_ID(COMP_RECO, LVL_REC, 201) ,(*rver)->xv, (*rver)->yv ,1);
	} else { 
          rhp_hfill(DQ_ID(COMP_RECO, LVL_REC, 202) ,(*rver)->xv, (*rver)->yv ,1);
	}
	
	
	if (diff < 100)
	  rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, 203) , float(igtar) , 1 );
	
	
	if ( igtar == 1 || igtar == 2 || igtar == 5 || igtar == 6){
	  dx = (*rver)->xv;
	  dy = (*rver)->yv - Vgtar->y;
	} else {
	  dx = (*rver)->xv - Vgtar->x;
	  dy = (*rver)->yv;
	}
	
	dz = (*rver)->zv - Vgtar->z;
	
	rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, (209+igtar)) , dx, 1 );
	rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, (219+igtar)) , dy, 1 );
	rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, (229+igtar)) , dz, 1 );
	
      }
    }
  }

  // show up
  ARTE_mark( "DQ_RECO_vertex_evnt 1", 0 );

  return 0;
}
