//             -*- Mode: C++ -*-
//
// util.h - general utility routines
//
// $Id: DQ_util.hh,v 1.1 2000/04/19 20:42:16 natasha Exp $

#ifndef _BFASTUTIL_H_
#define _BFASTUTIL_H_


// Dynamic cast template function - dynamically cast a base class pointer
//    to a derived class pointer.
//
// Syntax :
//   If Base and Derived are the class names of a base class and class
//   derived from Base, then
//
//    Derived * dyn_cast(Derived, Base *p)
//
//   e.g.
//      Derived *d;
//      Base *b;
//        ...
//      d = dyn_cast(Derived, b);
//
//   If there is an error, a 0 pointer is returned. Note that there are some
//   cases, particularly associated with multiply inherited classes, where
//   the cast would be legitimate, but causes a compile time error.

template <class T1, class T2>
T1 * dyn_cast_func( T1 *type, T2 *obj) {
     if (! obj->isA( T1::ClassID() )) return 0;
     T2 *b1 = (T1 *)0x10000;
     return (T1*) ((char *)obj - (char *)b1 + (char *)0x10000);
}
#define dyn_cast(A,B) dyn_cast_func((A *)0, B)

#endif
