
// *****************************************************************************
//
// source:    faMultiTimer.C
//
// type:      source code
//
// created:   23 Jun 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    
//
// called by: fainit_ fastop_
//
// purpose:   invoke rpm_receive/dispatch from time to time
//
// hints:     cannot be compiled with g++ -ansi
//
// changes:
// 23 Jun 2000 (AG) creation
// 27 Jun 2000 (AG) corrected printout
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <signal.h>

// ONLINE includes
#include "grr.h"
#include "rpm.h"
#include "rhp.h"

// ARTE includes
#include "farm/farm.hh"

// externals
extern int FADEB;

// prototypes
void faMultiTimerHdl( int sig );

//
// activate a multishot timer
//
int
faMultiTimerStart( int t_ms )
{
  // show up
  ARTE_mark( "faMultiTimerStart 0", 0 );

  int isec = t_ms / 1000;
  int usec = ( t_ms%1000 ) * 1000;
  struct itimerval time_preset;
  time_preset.it_interval.tv_sec  = isec;
  time_preset.it_interval.tv_usec = usec;
  time_preset.it_value.tv_sec     = 0;
  time_preset.it_value.tv_usec    = 10000;

  // signal action
  struct sigaction newact;
  newact.sa_flags = SA_RESTART;
  sigemptyset( &newact.sa_mask );

  newact.sa_handler = faMultiTimerHdl;
  if( sigaction( SIGALRM, &newact, NULL ) ){
    perror( "timer_start - sigaction" );
    ARTE_error( GRR_SERROR, "faMultiTimerStart sigaction", GRR_FISHY );
    return GRR_FISHY;
  }
  else {
    ARTE_error( GRR_SINFO, "faMultiTimerStart sigaction", 0 );
  }
  siginterrupt( SIGALRM, 1 );
  
  // Start the interval timer
  if( setitimer( ITIMER_REAL, &time_preset, NULL ) ){
    perror( "start_timer - setitimer" );
    ARTE_error( GRR_SERROR, "faMultiTimerStart setitimer", GRR_FISHY );
    return GRR_FISHY;
  }
  else {
    ARTE_error( GRR_SINFO, "faMultiTimerStart setitimer", 0 );
  }

  // show up
  ARTE_mark( "faMultiTimerStart 1", 0 );

  return GRR_OK;
}

//
// stop the multishot timer
//
int
faMultiTimerStop()
{
  // show up
  ARTE_mark( "faMultiTimerStop 0", 0 );

  // stop signal
  struct sigaction newact;
  newact.sa_handler = SIG_IGN;
  sigemptyset( &newact.sa_mask );
  newact.sa_flags = 0;
  struct itimerval time_preset = {{0,0},{0,0}};
  if( sigaction( SIGALRM, &newact, NULL ) ){
    perror( "stop_timer - sigaction" );
    ARTE_error( GRR_SERROR, "faMultiTimerStop", GRR_FISHY );
    return GRR_FISHY;
  }
  else {
    ARTE_error( GRR_SINFO, "", 0 );
  }

  // stop timer
  if( setitimer( ITIMER_REAL, &time_preset, NULL ) ){
    perror( "stop_timer - setitimer" );
    ARTE_error( GRR_SERROR, "faMultiTimerStop setitimer", GRR_FISHY );
    return GRR_FISHY;
  }
  else {
    ARTE_error( GRR_SINFO, "faMultiTimerStop setitimer", 0 );
  }

  // show up
  ARTE_mark( "faMultiTimerStop 1", 0 );
  
  // return
  return GRR_OK;
}

//
// signal handler function
//
void
faMultiTimerHdl( int sig )
{
  // show up
  ARTE_mark( "faMultiTimerHdl 0", 0 );

  if( FADEB ){
    cout << "faMultiTimerHdl: Caught SIGALRM sig=" << sig << endl; 
  }

  // check for messages
  while( rhp_wait_for_one_msg_and_dispatch( NO_BLOCK ) == GRR_OK );

  // show up
  ARTE_mark( "faMultiTimerHdl 1", 0 );
}
