#include "arte/arte.hh"
#include "grover/grover.hh" // - class defs, f.prototypes and global variables

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//

int GroverAddToRVER(GroverVertexList* vertices,
		    int type = 1) 
{ //
  // Type = 1 for primaries, 2 for secondaries
  //
  // NOT COMPLETE! only partial information filled- for instance, no
  // relation between this vertex and a single entry in RTRA representing
  // the 'vertex particle':
  // D*->whatever  is a vertex which should be written to RTRA and its 
  // constituent RTRAs are written into the RREL table BUT the D* itself
  // is not written to RTRA. This must change soon!
  //
  // M.Sang 05/06/00
  //

  if (vertices == 0) return -1;
  if (!(type == 1 || type == 2)) return -2;

  bool verbose = false;//(groverCuts.Verbose == 1);
  RVER vertex;
  RTRA track;

  for (GroverVertexList::iterator ivert = vertices->begin();
       ivert != vertices->end(); ++ivert) {
    
    // first change the coordinate system
    GroverVertex tempvtx = *ivert; // hopefully this copies the tracks correctly too :-(
    tempvtx.LocalToGlobal();

    // Simple RVER stuff first
    vertex.xv=tempvtx.X();   vertex.yv=tempvtx.Y();   vertex.zv=tempvtx.Z();
    for (int i=0;i<6;i++) vertex.cv[i] = tempvtx.Cov(i);
    vertex.nt   = tempvtx.particles.size();
    vertex.mass = tempvtx.Mass();
    vertex.merr = tempvtx.MassError();
    vertex.wire = tempvtx.Wire();
    //vertex.chi2 = tempvtx.Chi2();
    // Fill RVER row
    ArteTable<RVER>::push_back(vertex);


    // Next, fill the relations between RVER and RTRA (if possible)
    ArtePointer<RVER> pvertex(ArteTable<RVER>::size()); 
    int charge = 0;
    for (GroverParticleList::iterator ipar = ivert->particles.begin();
	 ipar != ivert->particles.end(); ++ipar) {
      charge += ipar->Charge();
      if (ipar->Type() == 1) {
	// This GroverParticle was made from an entry in RTRA. 
	// Good! we can fill the relation
	ArtePointer<RTRA> parent = ipar->ParentRTRA();
	if (parent != 0) {
	  if (verbose) cout << "*** GroverAddToRVER - setting relation" << endl;
	  RREL relation;
	  relation.chi  = ipar->Chi2();
	  relation.flag = 1;
	  ArteTable<RREL>::push_back(relation);
	  ArtePointer<RREL> prelation(ArteTable<RREL>::size());
	  createRelation(pvertex, prelation, &RVER::rrel);
	  createRelation(parent,  prelation, &RTRA::rrel);
	}
	else {
	  if (verbose) 
	    cout << "*** GroverAddToRVER - error: RTRA parent of a vertex particle "
		 << " was not set!" << endl;
	}
      }
    }


    // RTRA stuff

    // Primaries first
    if (type == 1) { 
      float mom = tempvtx.Momentum();
      if (mom == 0.0) track.pf = 0.0;
      else track.pf = (charge!=0) ? static_cast<float>(charge)/mom : 1.0/mom;
      track.xf = tempvtx.X();  track.yf = tempvtx.Y();  track.zf = tempvtx.Z(); // correct?!
      track.txf = tempvtx.SlopeX();  track.tyf = tempvtx.SlopeY();
      for (int i=0; i<15; i++) track.cvf[i] = tempvtx.Cov(i);
      track.chi = tempvtx.Chi2();
      track.ever= pvertex;
      track.fit = 0;
      if (track.chi==0.0) track.fit += -100;
      track.fle = track.flm = track.flr = track.flt 
	= track.hca = track.hin = track.hmu = track.hou 
	= track.hri = track.hsi = track.htr = 0;
      track.lee = track.leh = track.lemi = track.lmh 
	= track.lmmu = track.lre = track.lrk = track.lrmu 
	= track.lrp = track.lrpi = track.lte = track.lth = 0.0;
      track.topp = 0;
      //track.topp = pointer to TOPP row for a J/Psi etc. used for the 
      //constraint -> if not constrained fit , set to zero.
      ArteTable<RTRA>::push_back(track);
    }

    else if (type == 2) {
      float mom = tempvtx.Momentum();
      if (mom == 0.0) track.pf = 0.0;
      else track.pf = (charge!=0) ? static_cast<float>(charge)/mom : 1.0/mom;
      track.xf = tempvtx.X();  track.yf = tempvtx.Y();  track.zf = tempvtx.Z(); // correct?!
      track.txf = tempvtx.SlopeX();  track.tyf = tempvtx.SlopeY();
      for (int i=0; i<15; i++) track.cvf[i] = tempvtx.Cov(i);
      track.chi = tempvtx.Chi2();
      track.ever= pvertex;
      track.fit = 3;// Flagged as a long-lived decay
      if (track.chi==0.0) track.fit += -100;
      track.fle = track.flm = track.flr = track.flt 
	= track.hca = track.hin = track.hmu = track.hou 
	= track.hri = track.hsi = track.htr = 0;
      track.lee = track.leh = track.lemi = track.lmh 
	= track.lmmu = track.lre = track.lrk = track.lrmu 
	= track.lrp = track.lrpi = track.lte = track.lth = 0.0;
      track.topp = 0;
      //track.topp = pointer to TOPP row for a J/Psi etc. used for the 
      //constraint -> if not constrained fit , set to zero.
      ArteTable<RTRA>::push_back(track);
    }



  } // finished loop over vertex list

  return 0;
}
