#include <algorithm>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "grover/grover.hh"

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//

int GroverFillLikeRover(GroverParticleList* theList)
{
  //
  // Fills a GroverParticleList in the same way as Rover 1.07:
  // Use all RSEGs; use all RTRAs without a matching RSEG
  // replace RSEGs which *do* have a matching RSEG
  //

  bool verbose = false;


  // bail out if nothing to vertex with
  if (!(ArteTable<RSEG>::booked || ArteTable<RSEG>::booked())) return -1;

  int errorflag = 0;
  int nrseg = 0, nrtra = 0, ntracks = 0;
  const int ASIZE = 1000;


  // Fill particle list from all VDS RSEGs
  for (ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
       irseg != ArteTable<RSEG>::end(); ++irseg) {
    if (!((*irseg)->cmp & ~(Rsegc::vxd | Rsegc::bit16))
	&& ((*irseg)->nhit >= 
	    5
	    //groverCuts.RPTrackQualNH
	    )
	) {
      GroverParticle particle(irseg);
      theList->push_back(particle);
      ntracks++; 
      nrseg++;
    }
  }


  //Loop over particles
  int jj = 0;
  ArtePointer<RTRA> rtra_list[ASIZE], rtra_to_keep;
  for (GroverParticleList::iterator ipar = theList->begin();
       ipar != theList->end(); ++ipar) {
    ArtePointer<RSEG> vdsseg = ipar->ParentRSEG();
    ArtePointer<RTRA> rtra_which_wins = 0;
    bool erase_particle = false;
    // loop over rtras and keep one if it has the same rseg as this gp
    for (ArteTable<RTRA>::iterator irtra = ArteTable<RTRA>::begin();
	 irtra != ArteTable<RTRA>::end(); ++irtra) {
      // check rseg against particle and flag for erasure
      for(ArteRelationIterator<RSEG> irrel = (*irtra)->rseg.begin();
	  irrel != (*irtra)->rseg.end(); ++irrel) {
	if(!((*irrel)->cmp & ~(Rsegc::vxd | Rsegc::bit16))) {
	  //compare (*irrel) and vdsseg
	  if (*irrel == vdsseg) {
	    // the RTRA represents the same particle as the GP(rtra). keep.
	    rtra_to_keep = irtra;
	    erase_particle = true;
	  }
	}
      }
    }
    // now erase segment particle if an RTRA can be used
    if (erase_particle) {
      rtra_list[nrtra] = rtra_to_keep;
      nrtra++; nrseg--;
      theList->erase(ipar--);
    }
  }


  // add all the rtras
  for (int i = 0; i<nrtra; i++) {
    GroverParticle temp(rtra_list[i]);
    theList->push_back(temp);
  }


  if (theList->empty()) errorflag = -1;
  return errorflag;
}
