#include <iostream>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "grover/grover.hh"

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//


int GroverFindMCJpsi(GroverVertexList* jpsi_list)
{
  int electron_found = 0, positron_found = 0;
  ArtePointer<MTRA> mtra_electron, mtra_positron;    
  // Find e+ & e- in MTRA table
  for(ArteTable<MTRA>::iterator mtraIt = ArteTable<MTRA>::begin();
      mtraIt != ArteTable<MTRA>::end(); ++mtraIt){
    if (((*mtraIt)->topp->lund != 11) &&
	((*mtraIt)->topp->lund !=-11)) continue;     // not e- or e+
    if ((*mtraIt)->btra->topp->lund != 443) continue;// mother not J/psi
    
    if ((*mtraIt)->topp->lund !=-11){
      mtra_electron = (*mtraIt); 
      electron_found = 1;
    }
    if ((*mtraIt)->topp->lund != 11){
      mtra_positron = (*mtraIt); 
      positron_found = 1;
    }
  }
  
  if (!electron_found || !positron_found) return -1;
  
  // Find mothers
  ArtePointer<MVER> mver_electron = mtra_electron->bver; 
  ArtePointer<MVER> mver_positron = mtra_positron->bver; 
  
  if (mver_electron != mver_positron) return -2; // or store tracks and vertex
  
  ArtePointer<RSEG> rseg_electron = RSEGCausedBy(mtra_electron);
  ArtePointer<RSEG> rseg_positron = RSEGCausedBy(mtra_positron);
  
  if ((rseg_electron == 0) || (rseg_positron == 0)) return -3;
  
  // keep reco vertex
  GroverParticle rseg_eminus(rseg_electron);
  GroverParticle rseg_eplus(rseg_positron);
  float momem = sqrt(mtra_electron->px*mtra_electron->px +
		     mtra_electron->py*mtra_electron->py +
		     mtra_electron->pz*mtra_electron->pz  );
  float momep = sqrt(mtra_positron->px*mtra_positron->px +
		     mtra_positron->py*mtra_positron->py +
		     mtra_positron->pz*mtra_positron->pz  );
  rseg_eminus.SetOMomentum(momem);
  rseg_eplus.SetOMomentum(momep);
  GroverVertex rgv(&rseg_eminus);
  rgv.Add(&rseg_eplus);
  jpsi_list->push_back(rgv);
  
  // done
  return 0;

}
