#include <iostream>
#include <math.h>
#include "arte/arte.hh"
#include "grover/DumpFormatter.hh"
#include "grover/grover.hh"



//  GroverParticle class definition
//  -------------------------------
//

//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//

// pre-definitions
float   scal(float v1[], float v2[]);
void     res(float v1[], float v2[], float R[]);
void     sum(float v1[], float v2[], float S[]);
void  mult_n(float v1[], float n,    float M[]);



GroverParticle::GroverParticle()
  : xb(0.), yb(0.), zb(0.), axb(0.), ayb(0.),
    xe(0.), ye(0.), ze(0.), axe(0.), aye(0.),
    oxb(0.), oyb(0.), ozb(0.), oaxb(0.), oayb(0.),
    oxe(0.), oye(0.), oze(0.), oaxe(0.), oaye(0.),
    p(-999.), invp(-999.), mass(-999.0), nhit(-1), q(-999), type(-1),
    usedinprimary(0), usedinsecondary(0), index(0),
    parRTRA(0), parRSEG(0), parRCCL(0), parMTRA(0), 
    parGP(0), parGPL(0), parGV(0),
    inlocal(0), inglobal(0), wprim(0.), wsec(0.),
    dist(1e9), chi2(0.), constraint(-1)
{
}



GroverParticle::GroverParticle(const ArtePointer<RSEG> rseg)
  : oxb(rseg->xf), oyb(rseg->yf), ozb(rseg->zf), 
    oaxb(rseg->txf), oayb(rseg->tyf),
    oxe(rseg->xe), oye(rseg->ye), oze(rseg->ze), 
    oaxe(rseg->txe), oaye(rseg->tye),
    invp(fabs(rseg->pf)),
    mass(0.0), // <- note!
    nhit(rseg->nhit), 
    q(-999), type(2),
    usedinprimary(0), usedinsecondary(0), index(0),
    parRTRA(0), parRSEG(rseg), parRCCL(0), parMTRA(0), 
    parGP(0), parGPL(0), parGV(0),
    inlocal(0), inglobal(1), wprim(0.0), wsec(0.),
    dist(1e9), chi2(0.0), constraint(-1)
{ // Construct new instance from an entry in the RSEG table
  if (invp>0.0) op = 1.0/invp;
  else op = -999.0;
  p = op; // NOT FINISHED! REMEMBER ORIGINAL INVP AND NEW MEMBER FUNCTIONS !!!
  for (int i=0; i<15; i++) { ocovb[i]=rseg->cvf[i];  ocove[i]=rseg->cve[i]; }
  // copy original start and finish points onto the variable ones
  xb=oxb; yb=oyb; zb=ozb; axb=oaxb; ayb=oayb;
  xe=oxe; ye=oye; ze=oze; axe=oaxe; aye=oaye;
  for (int i=0; i<15 ;i++) { covb[i]=ocovb[i];  cove[i]=ocove[i]; }
}



GroverParticle::GroverParticle(const ArtePointer<RTRA> rtra)
  : oxb(rtra->xf), oyb(rtra->yf), ozb(rtra->zf), 
    oaxb(rtra->txf), oayb(rtra->tyf), 
    invp(rtra->pf), mass(0.0), nhit(rtra->hsi), type(1),
    usedinprimary(0), usedinsecondary(0), index(0),
    parRTRA(rtra), parRSEG(0), parRCCL(0), parMTRA(0),
    parGP(0), parGPL(0), parGV(0), 
    inlocal(0), inglobal(1), wprim(0.), wsec(0.),
    dist(1e9), chi2(0.), constraint(-1)
{ // Construct new instance from an entry in the RTRA table
  p = (invp!=0.0) ? (fabs(1.0/invp)) : 0.0;
  op = p;
  for (int i=0; i<15 ;i++) { ocove[i] = ocovb[i] = rtra->cvf[i];}
  if ((rtra->hin + rtra->hou + rtra->hsi) <= 0) q = 0; // no tracker hits
  else q = (rtra->pf<0) ? -1 : 1;
  // copy original start and finish points onto the movable ones
  xb=oxb; yb=oyb; zb=ozb; axb=oaxb; ayb=oayb;
  xe=oxe; ye=oye; ze=oze; axe=oaxe; aye=oaye;
  for (int i=0; i<15 ;i++) { covb[i]=ocovb[i];  cove[i]=ocove[i]; }
}



GroverParticle::GroverParticle(ArtePointer<MTRA> mtra)
  : mass(mtra->mass), nhit(0), type(4),
    usedinprimary(0), usedinsecondary(0), index(0),
    parRTRA(0), parRSEG(0), parRCCL(0), parMTRA(mtra),
    parGP(0), parGPL(0), parGV(0),
    inlocal(0), inglobal(1),
    wprim(0.0), wsec(0.0)
{ // Construct new instance from an entry in the MTRA table
  // *** NOT FINISHED!!!!!! ***
  // zero errors
  for (int i=0;i<21;i++) covb[i]=ocovb[i]=cove[i]=ocove[i]=0.0;
  p = sqrt(mtra->px*mtra->px + mtra->py*mtra->py + mtra->pz*mtra->pz);
  invp = (p>0) ? (1.0/p) : 0.0;
  // start point from parent MC vertex
  xb=oxb=mtra->bver->x;   yb=oyb=mtra->bver->y;   zb=ozb=mtra->bver->z;
  // start slopes
  axb = oaxb = (mtra->pz>0) ? (mtra->px/mtra->pz) : 0.0;   
  ayb = oayb = (mtra->pz>0) ? (mtra->py/mtra->pz) : 0.0;
  // end point from daughter MC vertex (if long-lived particle)
  if (mtra->leng > 0.0) {
    xe=oxe=oxb; ye=oye=oyb; ze=oze=ozb; axe=oaxe=oaxb; aye=oaye=oayb;
  }
  else {xe=oxe=oxb; ye=oye=oyb; ze=oze=ozb; axe=oaxe=oaxb; aye=oaye=oayb;}
  q = static_cast<int>(mtra->topp->chrg / 3.0);
  for (ArteRelationIterator<MIMP> imimp = mtra->mimp.begin();
       imimp != mtra->mimp.end(); ++imimp) {
    if ((*imimp)->gede->cmp == 1) {
      nhit += distance((*imimp)->hitb.begin(), (*imimp)->hitb.end());
    }
  }
}



GroverParticle::~GroverParticle(void)
{ // Destructor 
}



int GroverParticle::SetParent(ArtePointer<RTRA> rtra) {
  parRSEG=0; parRCCL=0; parMTRA=0; parGPL=0; parGP=0; parGV=0;
  parRTRA=rtra; type=1; return 0;
}

int GroverParticle::SetParent(ArtePointer<RSEG> rseg) {
  parRTRA=0; parRCCL=0; parMTRA=0; parGPL=0; parGP=0; parGV=0;
  parRSEG=rseg; type=2; return 0;
}

int GroverParticle::SetParent(ArtePointer<RCCL> rccl) {
  parRTRA=0; parRSEG=0; parMTRA=0; parGPL=0; parGP=0; parGV=0;
  parRCCL=rccl; type=3; return 0;
}

int GroverParticle::SetParent(ArtePointer<MTRA> mtra) {
  parRTRA=0; parRSEG=0; parRCCL=0; parGPL=0; parGP=0; parGV=0;
  parMTRA=mtra; type=4; return 0;
}

int GroverParticle::SetParent(GroverParticle* gp) {
  parRTRA=0; parRSEG=0; parRCCL=0; parMTRA=0; parGPL=0; parGV=0;
  parGP=gp; type=50; return 0;
}

int GroverParticle::SetParent(GroverParticleList* gpl) {
  parRTRA=0; parRSEG=0; parRCCL=0; parMTRA=0; parGP=0; parGV=0;
  parGPL=gpl; type=51; return 0;
}

int GroverParticle::SetParent(GroverVertex* gv) {
  parRTRA=0; parRSEG=0; parRCCL=0; parMTRA=0; parGPL=0; parGP=0;
  parGV=gv; type=52; return 0;
}



void GroverParticle::Dump() { // Dump private members to stdout
  cout << "*** Grover: Dump() of GroverParticle object" << endl;
  DumpFormatter Nice;
  Nice.DumpClassName("GroverParticle");
  Nice.DumpComment("Part of the Grover vertexing package");
  Nice.DumpComment("Please send questions about this class "
		    "to the Grover authors");
  Nice.DumpHeader();
  Nice.DumpPrint("index",&index);
  Nice.DumpPrint("xb",&xb);
  Nice.DumpPrint("yb",&yb);
  Nice.DumpPrint("zb",&zb);
  Nice.DumpPrint("xe",&xe);
  Nice.DumpPrint("ye",&ye);
  Nice.DumpPrint("ze",&ze);
  Nice.DumpPrint("axb",&axb);
  Nice.DumpPrint("ayb",&ayb);
  Nice.DumpPrint("axe",&axe);
  Nice.DumpPrint("aye",&aye);
  Nice.DumpPrint("covb",covb,15);
  Nice.DumpPrint("cove",cove,15);
  Nice.DumpPrint("oxb",&oxb);
  Nice.DumpPrint("oyb",&oyb);
  Nice.DumpPrint("ozb",&ozb);
  Nice.DumpPrint("oxe",&oxe);
  Nice.DumpPrint("oye",&oye);
  Nice.DumpPrint("oze",&oze);
  Nice.DumpPrint("oaxb",&oaxb);
  Nice.DumpPrint("oayb",&oayb);
  Nice.DumpPrint("oaxe",&oaxe);
  Nice.DumpPrint("oaye",&oaye);
  Nice.DumpPrint("ocovb",ocovb,15);
  Nice.DumpPrint("ocove",ocove,15);
  Nice.DumpPrint("p",&p);
  Nice.DumpPrint("invp",&invp);
  Nice.DumpPrint("type",&type);
  Nice.DumpPrint("nhit",&nhit);
  Nice.DumpPrint("q",&q);
  Nice.DumpPrint("usedinprimary",&usedinprimary);
  Nice.DumpPrint("inlocal",&inlocal);
  Nice.DumpPrint("inglobal",&inglobal);
  Nice.DumpPrint("wprim",&wprim);
  Nice.DumpPrint("wsec",&wsec);
  Nice.DumpFooter();  
}



int GroverParticle::Smear(const int alg = 1) {
  // Smearing of a particle could be done in several ways, all
  // of which should be kept inside the class. It's rather basic
  // and intrinsic to the concept of a particle. For the future...
  int errorflag = -1;
  if (type == 4) {
    cout << "*** GroverParticle::Smear(): function not yet available" << endl;
  }
  return errorflag;
}



int GroverParticle::PropagateBeginTo(float zplane = 0.) {
  // from Rover 1.05 primary finder
  if(zb == zplane) return 0; // May already be there!
  // distances to current beginning and original beginning
  float dz = zplane - ozb, tinydz= zplane - zb;

  if ((fabs(tinydz)< 5.)||(tinydz>0.0)) { // ivan's
    //if (fabs(tinydz) < 5. && zplane < 5.) { // mine

    // Simple propagation near vtx is OK (~ no material)
    xb  = xb + tinydz*axb;
    yb  = yb + tinydz*ayb;
    zb  = zplane;  
    covb[0] = covb[0] + tinydz * (tinydz*covb[5] + covb[3] + covb[3]);
    covb[1] = covb[1] + tinydz * (tinydz*covb[8] + covb[6] + covb[4]);
    covb[2] = covb[2] + tinydz * (tinydz*covb[9] + covb[7] + covb[7]);
    covb[3] = covb[3] + tinydz * covb[5]; 
    covb[4] = covb[4] + tinydz * covb[8]; 
    covb[6] = covb[6] + tinydz * covb[8];
    covb[7] = covb[7] + tinydz * covb[9];
    covb[10]= covb[8] + tinydz * covb[12];
    covb[11]= covb[9] + tinydz * covb[13];
  }
  else {
    if ((type != 2) || (zplane > 200.)) {
      // simple propagation only :-|
      // This is a fairly large distance and we would like to propagate
      // it correctly taking material into account, unfortunately it's
      // not a VDS RSEG, so we use a longer-armed version of the above...
      xb  = oxb + dz*oaxb;
      yb  = oyb + dz*oayb;
      zb  = zplane;  
      covb[0] = ocovb[0] + dz * (dz*ocovb[5] + ocovb[3] + ocovb[3]);
      covb[1] = ocovb[1] + dz * (dz*ocovb[8] + ocovb[6] + ocovb[4]);
      covb[2] = ocovb[2] + dz * (dz*ocovb[9] + ocovb[7] + ocovb[7]);
      covb[3] = ocovb[3] + dz * ocovb[5]; 
      covb[4] = ocovb[4] + dz * ocovb[8]; 
      covb[6] = ocovb[6] + dz * ocovb[8];
      covb[7] = ocovb[7] + dz * ocovb[9];
      covb[10]= ocovb[8] + dz * ocovb[12];
      covb[11]= ocovb[9] + dz * ocovb[13];
    }
    else {
      // This is an RSEG with momentum (?) and the distance is large.
      // Propagate taking material into account
      // make copies of params and cov
      float nxb=oxb,nyb=oyb,nzb=ozb,naxb=oaxb,nayb=oayb,ninvp=invp;
      float ncov[15]={ocovb[0], ocovb[1], ocovb[2], ocovb[3], ocovb[4],
		      ocovb[5], ocovb[6], ocovb[7], ocovb[8], ocovb[9],
		      ocovb[10],ocovb[11],ocovb[12],ocovb[13],ocovb[14]};
      GroverCatsExtrap(parRSEG, zplane, 
		       &nxb, &nyb, &nzb, &naxb, &nayb, &ninvp, ncov);
      xb=nxb,yb=nyb,zb=nzb,axb=naxb,ayb=nayb,invp=ninvp;
      for (int i=0;i<15;i++) covb[i] = ncov[i];
      zb = zplane;
      covb[10] += dz * covb[12];
      covb[11] += dz * covb[13];
    }
  }
  return 0;
}




int GroverParticle::PropagateEndTo(float zplane = 2000.) {
  cout << "*** GroverParticle::PropagateEndTo(): function not yet available" << endl;
  return -1;
}



int GroverParticle::GlobalToLocal(void) {
  // Covariance unchanged (not a bad approximation, in fact)
  if (inglobal && !inlocal) {
    float oldb[5], olde[5], newb[5], newe[5];
    float ooldb[5], oolde[5], onewb[5], onewe[5];
    oldb[0] = xb; oldb[1] = yb; oldb[2] = zb; oldb[3] = axb; oldb[4] = ayb;
    olde[0] = xe; olde[1] = ye; olde[2] = ze; olde[3] = axe; olde[4] = aye;
    ooldb[0] = oxb;  ooldb[1] = oyb; ooldb[2] = ozb; 
    ooldb[3] = oaxb;  ooldb[4] = oayb;
    oolde[0] = oxe;  oolde[1] = oye; oolde[2] = oze; 
    oolde[3] = oaxe;  oolde[4] = oaye;
    acGlobalToLocal(newb, oldb, 1);
    acGlobalToLocal(newe, olde, 1);
    acGlobalToLocal(onewb, ooldb, 1);
    acGlobalToLocal(onewe, oolde, 1);
    xb = newb[0]; yb = newb[1]; zb = newb[2]; axb = newb[3]; ayb = newb[4];
    xe = newe[0]; ye = newe[1]; ze = newe[2]; axe = newe[3]; aye = newe[4];
    oxb = onewb[0];   oyb = onewb[1]; ozb = onewb[2]; 
    oaxb = onewb[3]; oayb = onewb[4];
    oxe = onewe[0];   oye = onewe[1]; oze = onewe[2]; 
    oaxe = onewe[3]; oaye = onewe[4];
    inglobal = false; inlocal = true;
  }
  return 0;
}



int GroverParticle::LocalToGlobal(void) {
  // Covariance unchanged (not a bad approximation, in fact)
  if (inlocal && !inglobal) {
    float oldb[5], olde[5], newb[5], newe[5];
    float ooldb[5], oolde[5], onewb[5], onewe[5];
    oldb[0] = xb; oldb[1] = yb; oldb[2] = zb; oldb[3] = axb; oldb[4] = ayb;
    olde[0] = xe; olde[1] = ye; olde[2] = ze; olde[3] = axe; olde[4] = aye;
    ooldb[0] = oxb;  ooldb[1] = oyb; ooldb[2] = ozb;
    ooldb[3] = oaxb;  ooldb[4] = oayb;
    oolde[0] = oxe;  oolde[1] = oye; oolde[2] = oze;
    oolde[3] = oaxe;  oolde[4] = oaye;
    acLocalToGlobal(oldb, newb, 1);
    acLocalToGlobal(olde, newe, 1);
    acLocalToGlobal(ooldb, onewb, 1);
    acLocalToGlobal(oolde, onewe, 1); 
    xb = newb[0]; yb = newb[1]; zb = newb[2]; axb = newb[3]; ayb = newb[4];
    xe = newe[0]; ye = newe[1]; ze = newe[2]; axe = newe[3]; aye = newe[4];
    oxb = onewb[0];   oyb = onewb[1]; ozb = onewb[2];
    oaxb = onewb[3]; oayb = onewb[4];
    oxe = onewe[0];   oye = onewe[1]; oze = onewe[2];
    oaxe = onewe[3]; oaye = onewe[4];
    inlocal = false; inglobal = true;
  }
  return 0; 
}


int GroverParticle::ResetParameters(void)
{ // Reset all the track parameters to their original (constructed) values
  // BUT remember that a coordinate transformation changes these original 
  // values too. If you want the *really* original parameters, follow the
  // Parent pointer and construct a new particle (or transform back, although
  // I do not guarantee that this is a unitary transformation!!!!!!!!)
  xb=oxb; yb=oyb; zb=ozb;
  xe=oxe; ye=oye; ze=oze;
  axb=oaxb; ayb=oayb;
  axe=oaxe; aye=oaye;
  p = op;
  for (int i=0;i<21;i++)   {covb[i]=ocovb[i];  cove[i]=ocove[i];}
  return 0;
}




float GroverParticle::DistanceTo(GroverWire* wire){
  // 24.07.00 V.Kekelidze
  float Dist = 0.0;
  float ax = axb;
  float ay = ayb;
  float fact = sqrt(ax*ax+ay*ay+1);
  float dz = this->BeginZ() - wire->Z();

  if(wire->Horizontal() == 1){
    float dy = this->BeginY() - wire->Y();
    Dist = fabs(dy-dz*ay)*fact/(1+ay*ay);
  }
  else if(wire->Vertical() == 1){
    float dx = this->BeginX() - wire->X();
    Dist = fabs(dz*ax-dx)*fact/(1+ax*ax);
  }
  return Dist;
}



float GroverParticle::Chi2DistTo(GroverWire* wire) {
  // Simple-minded 1D chi2 assuming infinitely long wires! 
  // If you don't like it, please write something better ;-)
  PropagateBeginTo(wire->Z());
  float space=0., sigmasq=0.;
  if (wire->Horizontal() == 1) {
    space   = fabs(yb - wire->Y());
    sigmasq = fabs(covb[2]);
  }
  else if (wire->Vertical() == 1) {
    space   = fabs(xb - wire->X());
    sigmasq = fabs(covb[0]);
  }
    float dist = ( (sigmasq>0.) ? (space*space/sigmasq) : 10000.0);
    return dist;
}





float GroverParticle::DistanceTo(GroverParticle* part){
  // 20.07.00 D.Emeliyanov
  this->PropagateBeginTo(0.0);
  part->PropagateBeginTo(0.0);
  float x1=this->BeginX(),       x2=part->BeginX();
  float y1=this->BeginY(),       y2=part->BeginY();
  float ax1=this->BeginSlopeX(), ax2=part->BeginSlopeX();
  float ay1=this->BeginSlopeY(), ay2=part->BeginSlopeY();
  float xy = ax1*ay2-ay1*ax2;
  float sq_sum = (ax1-ax2)*(ax1-ax2)+(ay1-ay2)*(ay1-ay2)+xy*xy;
  float UP = (x2-x1)*(ay1-ay2)+(y2-y1)*(ax2-ax1);
  return  fabs(UP/sqrt(sq_sum));
}




float GroverParticle::Chi2DistTo(GroverParticle* part) {
  // 19.07.00   D.Emeliyanov, V.Kekelidze
  float oz1 = this->BeginZ(), oz2 = part->BeginZ();
  this->PropagateBeginTo(0.0);
  part->PropagateBeginTo(0.0);
  float x1=this->BeginX(),       x2=part->BeginX();
  float y1=this->BeginY(),       y2=part->BeginY();
  float ax1=this->BeginSlopeX(), ax2=part->BeginSlopeX();
  float ay1=this->BeginSlopeY(), ay2=part->BeginSlopeY();
  float xy = ax1*ay2 - ay1*ax2;
  float sq_sum = (ax1-ax2)*(ax1-ax2)+(ay1-ay2)*(ay1-ay2)+xy*xy;
  float UP = (x2-x1)*(ay1-ay2)+(y2-y1)*(ax2-ax1);
  float dist = fabs(UP/sqrt(sq_sum));
  if (dist > 1e6) return 1e9;
  float Cov[8][8];
  for(int i=0; i<8; i++) for(int j=i; j<8; j++) Cov[i][j]=Cov[j][i]=0.0;

  int k=0;
  for(int i=0; i<4; i++) {
    for(int j=0; j<=i; j++){
      Cov[i][j]     = Cov[j][i]     = this->BeginCov(k);
      Cov[i+4][j+4] = Cov[j+4][i+4] = part->BeginCov(k);
      k++;
    }
  }
  float Jac[8]={0.0};
  float sssum = sqrt(sq_sum);
  float sssum_times_sq_sum = sssum*sq_sum;
  Jac[0] = (ay2-ay2)/sssum;
  Jac[1] = (ax1-ax2)/sssum;
  Jac[2] = ((y1-y2)/sssum - UP*(ax1-ax2+ay2*xy))/sssum_times_sq_sum;
  Jac[3] = ((x2-x1)/sssum + UP*(ay1-ay2-ax2*xy))/sssum_times_sq_sum;
  Jac[4] = -Jac[0];
  Jac[5] = -Jac[1];
  Jac[6] = ((y2-y1)/sssum + UP*(ax2-ax1+ay1*xy))/sssum_times_sq_sum;
  Jac[7] = ((x1-x2)/sssum - UP*(ay2-ay1+ax1*xy))/sssum_times_sq_sum;
  float sum=0.0, sigma=0.0;
  for(int i=0; i<8; i++){
    sum = 0.0;
    for(int j=0; j<8; j++)  sum += Jac[j]*Cov[i][j];
    sigma += Jac[i]*sum;
  }
  this->PropagateBeginTo(oz1);
  part->PropagateBeginTo(oz2);
  return (sigma != 0.0) ? dist*dist/sigma : 1e9;
}





float GroverParticle::DistanceTo(GroverVertex *vert){
  //20.07.00 D.Emeliyanov, V.Kekelidze
  //closest spatial distance to a vertex
  float x0 = this->BeginX();
  float y0 = this->BeginY();      
  float ax = this->BeginSlopeX();
  float ay = this->BeginSlopeY();
  float xv = vert->X();
  float yv = vert->Y();
  float zv = vert->Z();
  float Sz = fabs(ax*ax + ay*ay + 1);
  float dx = x0-xv,    dy=y0-yv,     dz=this->BeginZ()-zv;
  float d1 = dy-dz*ay, d2=-dx+dz*ax, d3=dx*ay-dy*ax;
  float UP = d1*d1 + d2*d2 + d3*d3;
  return (UP >= 0.0) ? sqrt(UP/Sz) : 1e9;
}




float GroverParticle::Chi2DistTo(GroverVertex * vert){
  // 20.07.00 D.Emeliyanov, V.Kekelidze
  float x0 = this->BeginX();
  float y0 = this->BeginY();      
  float ax = this->BeginSlopeX();
  float ay = this->BeginSlopeY();
  float xv = vert->X();
  float yv = vert->Y();
  float zv = vert->Z();
  float Sz = ax*ax + ay+ay + 1;
  float dx=x0-xv,    dy=y0-yv,     dz=this->BeginZ()-zv;
  float d1=dy-dz*ay, d2=-dx+dz*ax, d3=dx*ay-dy*ax;
  float UP = d1*d1 + d2*d2 + d3*d3;
  float dist = sqrt(UP/Sz);
  if (dist>1e6) return 1e9;

  float Cov[7][7];
  for(int i=0; i<7; i++) for(int j=i; j<7; j++) Cov[i][j]=Cov[j][i]=0.0;
  int k=0;
  for(int i=0;i<4; i++) {
    for(int j=0;j<=i;j++){
      Cov[i][j] = Cov[j][i] = this->BeginCov(k);
      k++;
    }
  }
  k=0;
  for(int i=4;i<7; i++) {
    for(int j=4;j<=i;j++){
      Cov[i][j] = Cov[j][i] = vert->Cov(k);
      k++;
    }
  }
  float Jac[7]={0.0};
  Jac[0] = (ay*d3 - d2)/(Sz*dist);
  Jac[1] = (d1 - ax*d3)/(Sz*dist);
  Jac[2] = ((dz*d2 - dy*d3)*Sz - UP*ax) / (dist*Sz*Sz);
  Jac[3] = ((dx*d3 - dz*d1)*Sz - UP*ay) / (dist*Sz*Sz);
  Jac[4] = -Jac[0];
  Jac[5] = -Jac[1];
  Jac[6] = (ay*d1 - ax*d2)/(Sz*dist);
  float sum=0.0, sigma=0.0;
  for(int i=0; i<7; i++){
    sum=0;
    for(int j=0; j<7; j++)  sum += Jac[j]*Cov[i][j];
    sigma += Jac[i]*sum;
  }

  return (sigma != 0.0) ? dist*dist/sigma : 1e9;
}




float GroverParticle::DistanceTo(float x, float y, float z){
  //21.07.00 VK
  float x0 = this->BeginX();
  float y0 = this->BeginY();      
  float ax = this->BeginSlopeX();
  float ay = this->BeginSlopeY();
  float Sz = fabs(ax*ax + ay*ay + 1);
  float dx = x0-x,     dy=y0-y,      dz=this->BeginZ()-z;
  float d1 = dy-dz*ay, d2=-dx+dz*ax, d3=dx*ay-dy*ax;
  float UP = d1*d1 + d2*d2 + d3*d3;
  return sqrt(UP/Sz);
}



float GroverParticle::Chi2DistTo(float x, float y, float z){
  // 21.07.00
  float x0=this->BeginX();
  float y0=this->BeginY();      
  float ax=this->BeginSlopeX();
  float ay=this->BeginSlopeY();
  float Sz=ax*ax+ay+ay+1;
  float dx=x0-x, dy=y0-y, dz=this->BeginZ()-z;
  float d1=dy-dz*ay, d2=-dx+dz*ax, d3=dx*ay-dy*ax;
  float UP=d1*d1+d2*d2+d3*d3;
  float dist=sqrt(UP/Sz);
  if (dist>1e6) return 1e9;
  float Cov[7][7];
  for(int i=0; i<4; i++) for(int j=i; j<4; j++) Cov[i][j]=Cov[j][i]=0.0;
  int k=0;
  for(int i=0;i<4; i++) {
    for(int j=0;j<=i;j++){
      Cov[i][j] = Cov[j][i] = this->BeginCov(k);
      k++;
    }
  }
  k=0;
  float Jac[7]={0.0};
  Jac[0] = (ay*d3-d2)/(Sz*dist);
  Jac[1] = (d1-ax*d3)/(Sz*dist);
  Jac[2] = ((dz*d2-dy*d3)*Sz-UP*ax)/(dist*Sz*Sz);
  Jac[3] = ((dx*d3-dz*d1)*Sz-UP*ay)/(dist*Sz*Sz);

  float sum=0.0, sigma=0.0;
  for(int i=0; i<4; i++){
    sum=0;
    for(int j=0; j<4; j++){
      sum+=Jac[j]*Cov[i][j];
    }
    sigma+=Jac[i]*sum;
  }

  return (sigma != 0.0) ? dist*dist/sigma : 1e9;
}






/////////////////////////////////////////////////////////////////
//
//  Vector functions (V.K. 15/06/00)
//

inline float scal(float v1[],float v2[]){
  float scalar = 0.0;
  for(int i=0; i<3; i++) scalar+=v1[i]*v2[i];
  return scalar;
}

inline void res(float v1[],float v2[], float R[]){
  for(int i=0; i<3; i++) R[i] = v1[i] - v2[i];
} 

inline void sum(float v1[], float v2[], float S[]){
  for(int i=0; i<3; i++) S[i] = v1[i] + v2[i];
}

inline void mult_n(float v1[], float n, float M[]){
  for(int i=0; i<3; i++) M[i] = v1[i] * n;
}

