#include "arte/arte.hh"
#include "grover/grover.hh"

//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//

//
// C like Interface function to fit up to five RTRA or RSEG (overloaded)
// tracks to a vertex
// Mass constraint is possible. Just include desired mass in function call
// returns -1 in case of an error and 0 if ok
//
// S.Schaller 07/09/00
//

int GroverQuickFit(float& x,  float& y,  float& z,
		   float& dx, float& dy, float& dz, 
                   float& chi2, float mass=0.0,
		   ArtePointer<RTRA> rtra1=0, ArtePointer<RTRA> rtra2=0, 
		   ArtePointer<RTRA> rtra3=0, ArtePointer<RTRA> rtra4=0, 
		   ArtePointer<RTRA> rtra5=0)
{
  int code=-1;
  
  chi2=-999.;
  x   =-999.;
  y   =-999.;
  z   =-999.;
  dx  =-999.;
  dy  =-999.;
  dz  =-999.;
  
  if (rtra1!=0 && rtra2!=0) {
    GroverParticle particle1(rtra1);  
    GroverParticle particle2(rtra2);   
    GroverVertex gvt(&particle1,&particle2); // Construct a vertex from two particles
    if (rtra3!=0) {
      GroverParticle particle3(rtra3); 
      gvt.Add(&particle3);          // Add another particle to the vertex...
      if (rtra4!=0) { 
	GroverParticle particle4(rtra4); 
	gvt.Add(&particle4);
	if (rtra5!=0) {
	  GroverParticle particle5(rtra5); 
	  gvt.Add(&particle5);
	}
      }
    }

    code = 0;                         // more than one track included in the vertex 
    code = gvt.Fit(mass);             // fit tracks to a vertex     
    x  = gvt.X();                     // get x position of vertex...  
    y  = gvt.Y();
    z  = gvt.Z();
    dx  = (gvt.Cov(0) >= 0.0) ? sqrt(gvt.Cov(0)) : 1e10; // get position errors = sqrt of
    dy  = (gvt.Cov(2) >= 0.0) ? sqrt(gvt.Cov(2)) : 1e10; //  the diagonal elements of the
    dz  = (gvt.Cov(5) >= 0.0) ? sqrt(gvt.Cov(5)) : 1e10; //covariance matrix  
    chi2= gvt.Chi2();
  }

  else return -1;                     // not more than one track given
  
  return code;
}    

/////////////////////////////////////////////////////////////////////////////////


int GroverQuickFit(float& x,  float& y,  float& z,
		   float& dx, float& dy, float& dz, 
                   float& chi2, float mass=0.0,
		   ArtePointer<RSEG> rseg1=0, ArtePointer<RSEG> rseg2=0, 
		   ArtePointer<RSEG> rseg3=0, ArtePointer<RSEG> rseg4=0, 
		   ArtePointer<RSEG> rseg5=0)
{
  int code=-1;
  
  chi2=-999.;
  x   =-999.;
  y   =-999.;
  z   =-999.;
  dx  =-999.;
  dy  =-999.;
  dz  =-999.;
  
  if (rseg1!=0 && rseg2!=0) {
    GroverParticle particle1(rseg1);  
    GroverParticle particle2(rseg2);   
    GroverVertex gvt(&particle1,&particle2); // Construct a vertex from two particles
    if (rseg3!=0) {
      GroverParticle particle3(rseg3); 
      gvt.Add(&particle3);          // Add another particle to the vertex...
      if (rseg4!=0) { 
	GroverParticle particle4(rseg4); 
	gvt.Add(&particle4);
	if (rseg5!=0) {
	  GroverParticle particle5(rseg5); 
	  gvt.Add(&particle5);
	}
      }
    }

    code = 0;                         // more than one track included in the vertex 
    code = gvt.Fit(mass);             // fit tracks to a vertex     
    x  = gvt.X();                     // get x position of vertex...  
    y  = gvt.Y();
    z  = gvt.Z();
    dx  = (gvt.Cov(0) >= 0.0) ? sqrt(gvt.Cov(0)) : 1e10; // get position errors = sqrt of
    dy  = (gvt.Cov(2) >= 0.0) ? sqrt(gvt.Cov(2)) : 1e10; //  the diagonal elements of the
    dz  = (gvt.Cov(5) >= 0.0) ? sqrt(gvt.Cov(5)) : 1e10; //covariance matrix  
    chi2= gvt.Chi2();
  }

  else return -1;                     // not more than one track given
  
  return code;
}    
                          
