*CMZ :          07/03/96  17.42.08  by  Thomas Fuljahn
*CMZ :  1.00/01 08/05/95  21.05.44  by  Dominik Ressing
*CMZ :  1.00/00 03/01/95  12.21.24  by  Dominik Ressing
*CMZ :  0.01/03 19/09/94  12.12.23  by  Fedor Ratnikov
*CMZ :  0.01/02 13/09/94  00.42.12  by  Fedor Ratnikov
*CMZ :  0.01/01 25/07/94  15.34.24  by  Dominik Ressing
*-- Author :    Dominik Ressing   24/07/94
************************************************************************
*
* L1RUNSI
* written by Dominik Ressing, 24/07/94 14.11.00
*
* This routines action:
* Simulate one trigger RUN:
* IF pileup : execute pretriggers with current-time and do all jobs
*             till current-time + BX-duration
* ELSE      : execute pretriggers at current time and do all jobs
*             till no more left -> the event is processed.
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* ierr        out         error flag (set to zero if no error)
*
*
************************************************************************
c
      SUBROUTINE L1RUNSI(ierr)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1runpa.fi"
#include "l1si/l1jobdf.fi"
#include "l1si/l1mesdf.fi"
#include "l1si/l1prcdf.fi"
c
      INTEGER ierr,NextJob,NextTime,MessPo,last,
     &        iPrcPo,iType,iwait,itime,Nfif,iPre
      LOGICAL val
#include "l1si/l1mesdf.ff"
c
      ierr = 0                                  ! no error ( so far )
      L1RUNnextBX = L1RUNcutime + L1PARbxdis    ! set our time window
c
c     --- execute Pretrigger and submit new jobs
      DO iPre = 1,L1PARnpre                     ! use all defined pretriggers
         IF ( L1PARdetail.eq.0 ) THEN
            CALL l1prefa(iPre,ierr)             ! fast simulation
         ELSE
            CALL l1preha(iPre,ierr)             ! hardware simulation
         END IF
      END DO
c
      IF(L1MEBXtotal(L1RUNcuBX).LE.0)  THEN ! no pretrigger messages
         CALL l1usrev(L1RUNcuBX)            ! perform user actions
         CALL L1DATSC(L1RUNcuBX)            ! drop event
      ENDIF
c
      DO WHILE( (ierr.eq.0 ) .and.              ! break on error
     &          ( (L1RUNcutime.lt.L1RUNnextBX)  ! execute all within that BX time
     &        .or.(.not.L1PARpileup)))          ! or all for BX if no pile-up
c
         CALL l1jobex(NextJob,ierr)             ! next job to execute
c
         IF ( NextJob.eq.0 ) RETURN             ! no further jobs or error
c
c        --- get pointers
         MessPo = L1JOBstore(NextJob+L1JOBmess)
         last   = L1MEstor(MessPo+L1MElast)     ! last message update number
         iPrcPo = L1MEstor(MessPo+L1MEnextPC)   ! pointer to next processor
c
         CALL l1usrex(NextJob)                  ! user actions ...
         CALL l1jobdi(NextJob)                  ! disconnect from stack
c
c        --- call processor handling routine
         iType = L1PCstor(iPrcPo+L1PCtype)      !
         val = .true.                           ! job (message) still valid
         IF ( L1PCstor(iPrcPo+L1PCemul).eq.L1PCtteb ) THEN      ! test board
            CALL l1tebfa(NextJob,val,itime,ierr)     ! only fast simulation
         else

         IF ( iType.eq.L1PCttfu ) THEN          ! TFU
            IF ( L1PARdetail.eq.0 ) THEN
               CALL l1tfufa(NextJob,val,itime,ierr) ! fast simulation
            ELSE
               CALL l1tfuha(NextJob,val,itime,ierr) ! hardware simulation
            END IF
         ELSE IF ( iType.eq.L1PCttpm ) THEN     ! TPM
            IF ( L1PARdetail.eq.0 ) THEN
               CALL l1tpmfa(NextJob,val,itime,ierr) ! fast simulation
            ELSE
               CALL l1tpmha(NextJob,val,itime,ierr) ! hardware simulation
            END IF
         ELSE IF ( iType.eq.L1PCtMCU ) THEN     ! MCU
            IF ( L1PARdetail.eq.0 ) THEN
               CALL l1mcufa(NextJob,itime,ierr)     ! fast simulation
            ELSE
               CALL l1mcuha(NextJob,itime,ierr)     ! hardware simulation
            END IF
            val = .false.                           ! MCU terminates message processing
            CALL l1mesda(MessPO)                    ! deactivate message
         END IF
         END IF
         CALL l1usraf(NextJob,itime)             ! user actions ...
         L1PCstor(iPrcPo+L1PClast) = L1RUNcutime ! last processing start was now
         IF (val) THEN
            CALL l1jobrc(itime,NextJob)                   ! sort into stack at itime
         ELSE
            CALL l1jobco(NextJob,L1JOBsfree)              ! job into free chain
            L1JOBstore(NextJob+L1JOBflag) = L1JOBFLAGinva ! job in state ``invalid''
            IF ( iType.ne.L1PCtMCU ) THEN                 ! keep message for MCU always
               CALL l1usrdo(MessPo,val)                   ! user may want to keep mes.
               IF ( val ) THEN
                  CALL l1mesde(MessPO,L1MEDEusr) ! delete message + user actions
               END IF
            END IF
         END IF
c
      END DO
c
      RETURN
c
      END
