
// Boris Fominykh  16.12.1999
// member functions of class MuDbConfig

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbConfig.h"

//** Constructor */
MuDbConfig::MuDbConfig()
  :  MuDbTable::MuDbTable()
{
  Ini();
  strncpy(DBtable,"MuonConfig",11);
  strncpy(Desc,"A:cablename US:fed US:ncab",27);
  strncpy(Title,"Muon Cable Connections",23);
}

//** Destructor */
MuDbConfig::~MuDbConfig()
{
  MuDbConfig::Release();
}

//## Methods

//** Initialization */
void MuDbConfig::Ini()
{
  SizeS  =0;
  SizeMem=0;
  Name   = NULL;
  Fed    = NULL;
  Ncab   = NULL;
  SizeW  =0;
  NameW  = NULL;
  FedW   = NULL;
  NcabW  = NULL;
  return;
}
 
//** Read the table from DB into booked temporary space*/
int MuDbConfig::Read(int* NcabR)
{
  *NcabR=0;
  int ret;
// read the table from the file
  if(KTstatus == -3) {
    MuDbTable::setFile(getenv("MUON_CONFIG"));
    int ncw = 0;
    ret=MuDbConfig::ReadFile(&ncw);
    *NcabR=ncw;
    return ret;
  }
// open DB
  ret=MuDbTable::Open(0);
  if(ret > 0) return ret;
// fetch the table
  ret=MuDbTable::Fetch();
  if(ret > 0) return ret;
  SizeMem=Size+50;
  SizeS=Size;
// book the space
  Name = new char[10*SizeMem];
  if(Name == NULL) return 11;
  Fed  = new Word[SizeMem];
  if(Fed == NULL) {
    delete []Name;
    return 12;
  }
  Ncab = new Word[SizeMem];
  if(Ncab == NULL) {
    delete []Name;
    delete []Fed;
    return 13;
  }
// read the table from DB
  Nf=SkipTableField(X,1);
  int nel = GetTableField(X,TABLEascii,10*SizeMem,Name);
  if(nel <= 0) return 21;
  Nf=SkipTableField(X,2);
  nel = GetTableField(X,TABLEushort,SizeMem,Fed);
  if(nel <= 0) return 22;
  Nf=SkipTableField(X,3);
  nel = GetTableField(X,TABLEushort,SizeMem,Ncab);
  if(nel <= 0) return 23;
// close DB;
  MuDbTable::Close();
  *NcabR=SizeS;
  return 0;
}

//** Release the temporary space*/
void MuDbConfig::Release()
{
  if(SizeMem > 0) {
    delete []Name;
    delete []Fed;
    delete []Ncab;
    MuDbConfig::Ini();
  }
  return;
}

//** Print the table */
void MuDbConfig::Print()
{
  if(SizeMem < 1) {
    arteMsg << "MuDbConfig::Print - table in not read" << endl;
    arteMsg.warn();
    return;
  }

  ofstream fout("MuConfigDump.out");
  if (!fout) 
  {
    arteMsg << "MuDbConfig::Print - can not open file MuConfigDump.out";
    arteMsg << endl;
    arteMsg.warn();
    return;
  }

//  cout << "print table MuonConfig into file MuConfigDump.out" << endl;
//  cout << "DataBase: " << DBname << " Table: " << DBtable;
//  cout << " Title: <" << Title << "> " << endl;
//  cout << " Desc: <" << Desc << "> Size= " << SizeS;
//  cout << " Run " << Run << " Minor " << Minor << endl;

  fout << "DataBase: " << DBname << " Table: " << DBtable;
  fout << " Title: <" << Title << "> " << endl;
  fout << " Desc: <" << Desc << "> Size= " << SizeS;
  fout << " Run " << Run << " Minor " << Minor << endl;

  char cablename[10];
  char* pname = Name;
  char* M = "M";
  for(int i=0; i<SizeMem; i++) {
    strncpy(cablename,pname,10);
    cablename[9]='\0';
    char chi = cablename[0];
    if(chi != *M) break;
    pname+=10;
    Word *pfed = Fed+i;
    Word *pncab= Ncab+i;

    fout << "  #" << i+1 << ": ";
    fout << cablename << " FED " << hex << *pfed;
    fout << " Cable " << dec << *pncab << endl; 
  }

  return;
}

//** Get all cable attribures*/
int MuDbConfig::GetCabl(int* ncb,char* cable,Word* fed,Word* ncab)
{
  *fed = 0;
  *ncab= 0;
  if(SizeMem < 1) {
    arteMsg << "MuDbConfig::GetCabl - table in not read" << endl;
    arteMsg.warn();
    return 1;
  }
  if(*ncb < 0 || *ncb > SizeS-1) {
    arteMsg << "MuDbConfig::GetCabl - cable number " << *ncb;
    arteMsg << " is OUT of range 0-" << SizeS-1 << endl;
    arteMsg.warn();
    return 2;
  }
  char* pname = Name+10*(*ncb);
  Word* pfed = Fed+(*ncb);
  Word* pncab= Ncab+(*ncb);
  char* M = "M";
  char www[10];
  strncpy(www,pname,10);
  if(www[0] != *M) {
    arteMsg << "MuDbConfig::GetCabl - cable name " << www;
    arteMsg << " for number " << *ncb << " is strange" << endl;
    arteMsg.warn();
    return 3;
  }
  *fed = *pfed;
  *ncab= *pncab;
  strncpy(cable,www,10);
  return 0;
}

//** Get the cable connection*/
int MuDbConfig::GetConn(char* cable,Word* fed,Word* ncab)
{
  *fed = 0;
  *ncab= 0;
  if(SizeMem < 1) {
    arteMsg << "MuDbConfig::GetConn - table in not read" << endl;
    arteMsg.warn();
    return 1;
  }

  char cablin[10];
  strncpy(cablin,cable,10);
  char* pix = "x";
  char chi = cablin[1];
  if(chi == *pix) cablin[8]=' ';
  cablin[9]='\0';
  char* pname = Name;
  Word* pfed = Fed;
  Word* pncab= Ncab;
  char* M = "M";
  char www[10];
  for(int i=0; i<SizeMem; i++) {
    strncpy(www,pname,10);
    char chi = www[0];
    if(chi != *M) break;

    if(!strcmp(pname,cablin)) {
      *fed = *pfed;
      *ncab= *pncab;
      return 0;
    }
    pname+=10;
    pfed +=1;
    pncab+=1;
  }
  arteMsg << "MuDbConfig::GetConn - " << cable << " is NOT found" << endl;
  arteMsg.warn();
  return 2;
}

//** Get the cable name*/
int MuDbConfig::GetName(Word* fed,Word* ncab,char* cable)
{
  if(SizeMem < 1) {
    arteMsg << "MuDbConfig::GetName - table in not read" << endl;
    arteMsg.warn();
    return 1;
  }
  char* pname = Name;
  Word* pfed = Fed;
  Word* pncab= Ncab;
  char* M   = "M";
  char www[10];
  for(int i=0; i<SizeMem; i++) {
    strncpy(www,pname,10);
    char chi = www[0];
    if(chi != *M) break;
    if(*fed == *pfed && *ncab == *pncab) {
      strncpy(cable,pname,10);
      return 0;
    }
    pname+=10;
    pfed +=1;
    pncab+=1;
  }
  arteMsg << "MuDbConfig::GetConn - FED " << hex << *fed;
  arteMsg << " Ncab " << dec << *ncab << " is NOT found" << endl;
  arteMsg.warn();
  return 2;
}

//** Create the table in DB*/
int MuDbConfig::Create()
{
  int ncab=0;
  int rt=ReadFile(&ncab);
  if(rt != 0) return rt;
  if(ncab <= 0) return 50;
  int ret=Write();
  return ret;
}

//** Read the table from ascii-file into temporary space*/
int MuDbConfig::ReadFile(int* NcabR)
{
  *NcabR=0;
  FILE *hl = fopen(FileName, "r");
  if(!hl) {
    arteMsg << "MuDbConfig::Create - no input File" << endl;
    arteMsg.warn();
    return 1;
  }

// loops over lines: max 10000 lines
  char* M = "M";
  char* tub = "t";
  char* pad = "p";
  char* pix = "x";
  char line[100];
  for(int il=0; il<10000; il++) {
// read the line
    if(fgets(line,sizeof(line),hl) == NULL) {
      fclose(hl);
      goto goodexit;
    }
    int fedr  =0;
    int ncabr =0;
    Word fed  =0;
    Word ncab =0;
    char cablname[10];
    if (sscanf(line, "FED %x   CABLE %d   NAME %s " ,
               &fedr,&ncabr,cablname) == 3) {
      char chi = cablname[0];
      char ch2 = cablname[1];
      if(chi == *M && (ch2 == *tub || ch2 == *pad || ch2 == *pix))
      {
        if(ch2 == *pix) cablname[8]=' ';
        cablname[9]='\0';
        fed  = Word(fedr);
        ncab = Word(ncabr);

//      cout << "#" << dec << il << ":  " << cablname << " fed ";
//      cout << hex << fed << " Ncab " << dec << ncab << endl;

        Add(cablname,fed,ncab);
      }
    }
  }
goodexit:
  if(SizeW > 0) {
    *NcabR = SizeW;
    SizeMem= SizeW;
    SizeS  = SizeW;
    Name   = NameW;
    Fed    = FedW;
    Ncab   = NcabW;
    return 0;
  }
  else return 50;
}

//** Add one line to the temporary arrays */
int MuDbConfig::Add(char* cablname,Word fed,Word ncab)
{
  if(SizeW%SizeWQ == 0) {
// book the space
    int size = SizeWQ; 
    if(SizeW > 0) size += SizeW; 
    char* name = new char[10*size];
    if(name == NULL) return 1;
    Word* fedw = new Word[size];
    if(fedw == NULL) {
      delete []name;
      return 2;
    }
    Word* ncabw = new Word[size];
    if(ncabw == NULL) {
      delete []fedw;
      delete []name;
      return 3;
    }
    if(SizeW > 0) {
// rewrite old into new
      for(int i=0; i<SizeW; i++) {
        strncpy(name+10*i,NameW+10*i,10);
        *(fedw+i)=*(FedW+i);
        *(ncabw+i)=*(NcabW+i);
      }
      delete []NameW;
      delete []FedW;
      delete []NcabW;
    }
    NameW=name;
    FedW =fedw;
    NcabW=ncabw;
  }
// add the last line
  strncpy(NameW+10*SizeW,cablname,10);
  *(FedW+SizeW)=fed;
  *(NcabW+SizeW)=ncab;
  SizeW += 1;
  return 0;
}

//** Write the table to DB */
int MuDbConfig::Write()
{
  if(SizeW <= 0) return 1;
// open DB
  int ret=MuDbTable::Open(1);
  if(ret > 0) return ret;
// write the table to DB
  SkipTableField(X,1);
  int ret1=PutTableField(X,TABLEascii,10*SizeW,NameW);
  int ret2=PutTableField(X,TABLEushort,SizeW,FedW);
  int ret3=PutTableField(X,TABLEushort,SizeW,NcabW);
  int ret4=StoreTable(X,T,0);
// delete the temporary space
  delete []NameW;
  delete []FedW;
  delete []NcabW;
  MuDbConfig::Ini();
// close DB;
  MuDbTable::Close();
//
  if(ret1 > 0 && ret2 > 0 && ret3 > 0 && ret4 > 0) return 0;
  if(ret1 <= 0) return 11; 
  if(ret2 <= 0) return 12; 
  if(ret3 <= 0) return 13; 
  if(ret4 <= 0) return 14; 
  return 15;
}
