
// Boris Fominykh  22.12.1999

// print the muon detector geometry from DataBase NEDE and NESL
// classes MuDbNede and MuDbNesl are used for communication with Mizzi DB
// classes MuNede and MuNesl are the containers for NEDE, NESL elements

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "murec/MuDbNesl.h"
#include "murec/MuDbNede.h"

extern "C" {
int MuDbGeoPrint(int RUN, char* DBname) {
//*************************************************//

  ofstream fout("MuDbGeoPrint.out");
  if (!fout) 
  {
    cout << "MuDbGeoPrint: can not open the output file MuDbGeoPrint.out" << endl;
    return 1;
  }
  int ret;

  MuNesl   lnesl;  
  MuDbNesl Nesl;  
  Nesl.setDBname(DBname);
  Nesl.setRun(RUN);
  Nesl.setMinor(0);
  ret=Nesl.Read();
  if(ret != 0) {
    fout << "MuDbGeoPrint: NESL table is NOT read ret= " << ret;;
    fout << " DBname= " << DBname << " RUN= " << RUN << endl;
    return 2;
  }

  MuNede   lnede;  
  MuDbNede Nede;  
  Nede.setDBname(DBname);
  Nede.setRun(RUN);
  Nede.setMinor(0);
  ret=Nede.Read();
  if(ret != 0) {
    fout << "MuDbGeoPrint: NEDE table is NOT read ret= " << ret;;
    fout << " DBname= " << DBname << " RUN= " << RUN << endl;
    return 3;
  }

// loops over superlayers
  for(int i=0; i<8; i++) {
    int j=i/2;
// superlayer key
    int neslkey = 18+10*j+i-2*j;
    ret=Nesl.Get(&neslkey,&lnesl);
    if(ret != 0) {
      fout << "MuDbGeoPrint: no NESL for key " << neslkey << endl;
    }
    else {

// superlayer name
      char namsl[5];
      strncpy(namsl,&(lnesl.name[0]),5);
      namsl[4]='\0';

// print the superlayer information
      fout << namsl << " inesl " << lnesl.inesl;
      fout << " key " << neslkey << " cmp " << lnesl.cmp;
      fout << " lay " << lnesl.lay;
      fout << " nsl " << lnesl.nsl;
      fout << " npl " << lnesl.npl;
      fout << " fpl " << lnesl.fpl;
      fout << " nwl " << lnesl.nwl;
      fout << " fwl " << lnesl.fwl;
      fout << " ltyp " << lnesl.ltyp;
      fout << " z1 " << lnesl.z1;
      fout << " z2 " << lnesl.z2 << endl;

// parameters of chamber structure
      int nxmem[2][3];
      int nx99m[2][3];
      for(int i=0; i<2; i++) {
        for(int j=0; j<3; j++) {
          nxmem[i][j]=0;
          nx99m[i][j]=0;
        }
      }
      int msl = lnesl.nsl;
      int cmp = lnesl.cmp;
      int laym = lnesl.lay;
      if(cmp == 8 && laym > 0) {
        for(int l=0; l<laym; l++) {
          fout << "lay " << l;
          for(int i=0; i<2; i++) {
            nxmem[i][l]=lnesl.chamb[2*l+i];
            nx99m[i][l]=lnesl.nx99[2*l+i];
            fout << " nx" << i << " " << nxmem[i][l];
            fout << " nx99-" << i << " " << nx99m[i][l];
          }
          fout << endl;
        }
      }

// cross-check
      int nvol=lnesl.npl;
      int nvl=0;
      if(cmp == 8 && laym > 0) {
        for(int l=0; l<laym; l++) {
          for(int i=0; i<2; i++) {
            nvl+=2*nxmem[i][l];
            nvl+=2*nx99m[i][l];
          }
        }
      }
      if(cmp == 9) nvl=laym;
      if(nvl == nvol) fout << "cross-check OK, nvol= " << nvol << endl;
      else fout << "cross-check: nvol " << nvol << " nvl " << nvl << endl;

// loops over cmp=8 nede elements
      if(cmp == 8 && laym > 0) {
        for(int lay=0; lay<laym; lay++) {
          for(int ud=0; ud<2; ud++) {
            if(nxmem[ud][lay] > 0) {
              for(int nx=0; nx<nxmem[ud][lay]+1; nx++) {
                int nxx=0;
                if(nx < nxmem[ud][lay]) nxx=nx+1;
                if(nx == nxmem[ud][lay] && nx99m[ud][lay] == 1) nxx=99;
                if(nxx > 0) {
                  for(int sd=0; sd<2; sd++) {
// key word
  int nedekey = 100000*msl+10000*(lay+1)+1000*ud+10*nxx+sd;
                    ret=Nede.Get(&nedekey,&lnede);
                    if(ret != 0) {
      fout << "MuDbGeoPrint: no NEDE for key " << nedekey << endl;
                    }
                    else {
// chamber name
                    char namde[9];
                    strncpy(namde,&(lnede.hwc[0]),9);
                    namde[8]='\0';
                    char namvl[5];
                    strncpy(namvl,&(lnede.gvol[0]),4);
                    namvl[4]='\0';
// print the gede information
      fout << msl << "-" << lay+1 << " : ";
      fout << namsl << " - " << namde << " " << namvl << " key " << nedekey;
      fout << " neslkey " << lnede.neslkey;
      fout << " inede " << lnede.inede;
      fout << " cmp " << cmp << " ud " << ud;
      fout << " nx " << nxx << " sd " << sd;
      fout << " z1 " << lnede.zmin << " z2 " << lnede.zmax;
      fout << endl;
      fout << " x1 " << lnede.xmin << " x2 " << lnede.xmax;
      fout << " y1 " << lnede.ymin << " y2 " << lnede.ymax;
      fout << endl;
                    }
                  }
                }
              }
            }
          }
        }
      }

// loops over cmp=9 nede elements
      if(cmp == 9 && laym > 0) {
        for(int qu=0; qu<laym; qu++) {
// key word
          int nedekey = 1000*msl+100*(qu+1)+cmp;
          ret=Nede.Get(&nedekey,&lnede);
          if(ret != 0) {
      fout << "MuDbGeoPrint: no NEDE for key " << nedekey << endl;
          }
          else {
// chamber name
            char namde[6];
            strncpy(namde,&(lnede.hwc[0]),6);
            namde[5]='\0';
// print the gede information

      fout << namsl << " - " << namde;
      fout << " inede " << lnede.inede;
      fout << " cmp " << cmp << " key " << nedekey;
      fout << " msl " << msl << " qu " << qu;
      fout << " x1 " << lnede.xmin << " x2 " << lnede.xmax;
      fout << " y1 " << lnede.ymin << " y2 " << lnede.ymax;
      fout << " z1 " << lnede.zmin << " z2 " << lnede.zmax;
      fout << endl;

          }
        }
      }
    }
  }
  return 0;
}
}
