//
// Draw SLT tracks and RoI
//
// Alexander Lanyov         August 2000
//
#include <stdio.h>
#include <arte/arte.hh>
#include "arte/RSEG_cmp.hh"
#include "farm/farm.hh"

extern "C" {
#include "sltinfo.h"
}

#include "pris/prism_higz.h"
// updated version of cats/include/higz.h
// old version is in $HBROOT/VDS/pro/include/higz.h
// different version is in $HBROOT/BEE/BEE-0-82/include/Cern/higz.hh

extern "C" void dipro_(float *x, float *xwin, float *ywin);
extern "C" void igqwk_(int *, char *, float *, long int);

extern "C" void dislt_ () {

  //------------------------------------------------------ Basic SLT info
  int daqcode=faDDAQFCS_tcode();
  
  // Detailed trigger definition is given at:
  // http://www-hera-b.desy.de/subgroup/shiftinfo/sub/DAQSHI.html

  if (daqcode==9) return; /* No SLT information in non triggered events */

  if (SLTReadDSLT()!=SLT_OK) {
    cout << " === dislt: No SLT information" << endl;
    return;
  }
  
  SLT_STATUS stat;
  SLT_TRIGGER trig;
  SLT_TRACK track;
  SLT_SEEDTRACK seed;
  SLT_SILITRACK sili;
  SLT_REFITTRACK refit;

  if (SLTStatus(&stat)!=SLT_OK) {
    cout <<" === dislt: No status information"<<endl;
    return;
  }
  if (stat.status&SLTEVENT_BYPASS) {
    cout <<" === dislt: Event was a SLT BYPASS event"<<endl;
    if (!(stat.status&SLTEVENT_ACCEPT)) return;
  }
  if (!(stat.status&SLTEVENT_ACCEPT)) {
    cout <<" === dislt: Event rejected in SLT"<<endl;
    return;
  }

  for(int trigger=0;trigger<stat.ntriggers;trigger++) {
    if (SLTTrigInfo(trigger,&trig)!=SLT_OK) {
      cout<<" === dislt: Error getting info on trigger number "<<trigger<<endl;
      continue;
    }
    if (trig.status==SLTTRIG_REJECT) {
      // cout << endl <<" === dislt: Trigger "<<trigger<<" rejected"<<endl;
      //continue;
    }
    for(int tra=0;tra<trig.ntracks;tra++) {
      if (SLTTrackInfo(trig.trackindex[tra],&track)!=SLT_OK) {
	cout<<" === dislt: Error get info on track "
	    << trig.trackindex[tra] << endl;
	continue;
      }
      if(track.status==SLTTRACK_ERROR) continue;
      
      if (SLTSeedTrackInfo(trig.trackindex[tra  ],&seed)!=SLT_OK) {
	cout<<" === dislt: Error getting seed track for track"
	    << trig.trackindex[tra] << endl;
	continue;
      }

      //cout << " seed.z1 = " << seed.z1 << endl;
      //cout << " seed.z2 = " << seed.z2 << endl;

      if(track.status!=SLTTRACK_ACCEPT) {
	//cout<<" === dislt: track " << trig.trackindex[tra]
	//    << " was not accepted!" << endl;
	continue; // if we continue, meaningless tracks are drawn
      }
      
      //------------------------------------------------- Drawing
      int nt_detzg = 52; // NT for detector objects
      iselnt(nt_detzg);

      //      isclip(1);

      //----------- Draw Seed RoI
      float z1 = seed.z1;
      float z2 = seed.z2;

      extern struct {
	int ntzg, iviewzg, iframzg, idirzg; // and so on...
      } zgraf_;

      int v = zgraf_.iviewzg;
      float zwin1 = -1000.;
      float zwin2 = 10000.;
      if (v==1 || v==2 || (v>=7 && v<=10)) {
	float wn[4];
	int iwkid=1;
        //igqwk(iwkid,"NTWN",wn);
	igqwk_(&iwkid, "NTWN", wn, 4);
        if (zwin1 < wn[0]) zwin1 = wn[0];
        if (zwin2 > wn[1]) zwin2 = wn[1];
	//cout << " iviewzg = " << v << " zwin1 = " << zwin1 << ", zwin2 = " << zwin2 << endl;
      }
      if (z1 < zwin1) z1 = zwin1;
      if (z2 > zwin2) z2 = zwin2;

      float x1[3],x2[3],xx1[3],xx2[3];
      float xwin[5],ywin[5];

      x1[0] = seed.x1 + (seed.x2-seed.x1)*(z1-seed.z1)/(seed.z2-seed.z1);
      x1[1] = seed.y1 + (seed.y2-seed.y1)*(z1-seed.z1)/(seed.z2-seed.z1);
      x1[2] = z1;
      x2[0] = seed.x1 + (seed.x2-seed.x1)*(z2-seed.z1)/(seed.z2-seed.z1);
      x2[1] = seed.y1 + (seed.y2-seed.y1)*(z2-seed.z1)/(seed.z2-seed.z1);
      x2[2] = z2;

#if 0
      dipro_(x1, &xwin[0], &ywin[0]);
      dipro_(x2, &xwin[1], &ywin[1]);
      igset("LWID",2.);
      igset("PLCI",7.);
      ipl(2, xwin, ywin);
#endif

      igset("LWID",1.);
      igset("LTYP",3.);
      igset("PLCI",7.);
      for (int ix=-1; ix<=1; ix+=2) {
	for (int iy=-1; iy<=1; iy+=2) {
	  xx1[0] = x1[0] + ix*seed.x1_err;
	  xx1[1] = x1[1] + iy*seed.y1_err;
	  xx1[2] = x1[2];
	  xx2[0] = x2[0] + ix*seed.x2_err;
	  xx2[1] = x2[1] + iy*seed.y2_err;
	  xx2[2] = x2[2];
	  dipro_(xx1, &xwin[0], &ywin[0]);
	  dipro_(xx2, &xwin[1], &ywin[1]);
	  ipl(2, xwin, ywin);
	}
      }
      
      for (int iz=0; iz<2; iz++) {
	int i=0;
	for (int ix=-1; ix<=1; ix+=2) {
	  for (int iy=-1; iy<=1; iy+=2) {
	    if (iz==0) {
	      xx1[0] = x1[0] + ix*seed.x1_err;
	      xx1[1] = x1[1] + iy*seed.y1_err;
	      xx1[2] = x1[2];
	    } else {
	      xx1[0] = x1[0] + ix*seed.x2_err;
	      xx1[1] = x1[1] + iy*seed.y2_err;
	      xx1[2] = x1[2];
	    }
	    int k=i;         // i=0,1,2,3
	    if (k>1) k=5-i;  // k=0,1,3,2
	    dipro_(xx1, &xwin[k], &ywin[k]);
	    i++;
	  }
	}
	xwin[4] = xwin[0];
	ywin[4] = ywin[0];
	ipl(5, xwin, ywin);
       }
      igset("LTYP",1.);

      //----------- Draw refit line
      //cout << "calling SLTRefitTrackInfo(trig.trackindex[" << tra << "])" << endl;
      if (SLTRefitTrackInfo(trig.trackindex[tra],&refit)!=SLT_OK) {
	cout<<" === dislt: Error getting Refit track for track"
	    << trig.trackindex[tra]<<endl;
	continue;
      }
      else {
	x1[0] = refit.bx + refit.tx * z1;
	x1[1] = refit.by + refit.ty * z1;
	x1[2] = z1;
	x2[0] = refit.bx + refit.tx * z2;
	x2[1] = refit.by + refit.ty * z2;
	x2[2] = z2;
	
	dipro_(x1, &xwin[0], &ywin[0]);
	dipro_(x2, &xwin[1], &ywin[1]);
	igset("LWID",2.);
	igset("PLCI",2.);
	igset("LTYP",1.);

	ipl(2, xwin, ywin);

#if 0
	cout << endl << "Refit " << endl;
	cout << "xwin[0] = " << xwin[0] << endl;
	cout << "xwin[1] = " << xwin[1] << endl;
	cout << "ywin[0] = " << ywin[0] << endl;
	cout << "ywin[1] = " << ywin[1] << endl;
#endif

	//igraph(2, xwin, ywin, "L");

      }

      //----------- Draw Sili line
      //cout << "calling SLTSiliTrackInfo(trig.trackindex[" << tra << "])" << endl;
      if (SLTSiliTrackInfo(trig.trackindex[tra],&sili)!=SLT_OK) {
	cout<<" === dislt: Error getting L2sili track for track"
	    << trig.trackindex[tra]<<endl;
      }
      else {
	z1 = seed.z1;
	if (z1<-10. || z1>0.) z1 = -5.;
	z2 = 227.;
	if (z1 < zwin1) z1 = zwin1;
	if (z2 > zwin2) z2 = zwin2;

	x1[2] = z1;
	x1[0] = sili.bx + sili.tx * z1;
	x1[1] = sili.by + sili.ty * z1;
	
	x2[2] = z2;
	x2[0] = sili.bx + sili.tx * z2;
	x2[1] = sili.by + sili.ty * z2;
	
	dipro_(x1, &xwin[0], &ywin[0]);
	dipro_(x2, &xwin[1], &ywin[1]);
	igset("LWID",2.);
	igset("PLCI",2.);
	ipl(2, xwin, ywin);
      }

      igset("LWID",1.);
      igset("PLCI",1.);

      // To do: draw covariance matrix
      //SLT_SILITRACKCOV cov;
      //if (SLTSiliTrackCovInfo(trig.trackindex[tra],&cov)!=SLT_OK) {
      //   cout<<" === dislt: Error getting covariance matrix for track"
      //       << trig.trackindex[tra]<<endl;
      //   continue;
      //}

      iuwk(0,1); // Flush buffer
    }
  }

  return;
}
