// 
//  Author:  Rainer Mankel                     Created:  07-Mar-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerDomWin

#include <math.h>
#include "rang/DomWin.hh"
#include "rang/ProxyWin.hh"
#include "rang/ProxyLay.hh"

rangerDomWin::rangerDomWin(rangerDomain* rd,const int dCmp,const int dSec){
  
  Dom = rd;
  Cmp = dCmp;
  nShad = 0;
  for (int m=0; m<=2; ++m) {
    maxLsn[m] = 0;
    firstLay[m] = 0;
    lastLay[m] = 0;
  }
  entranceLay = 0;
  exitLay = 0;
  Sec = dSec;

  xMin = 0;
  xMax = 0;
  yMin = 0;
  yMax = 0;
  zMin = 0;
  zMax = 0;
};

void rangerDomWin::accum(const ArtePointer<GEDE> ge){

  float xMi = 0;
  float yMi = 0;
  float zMi = 0;
  float xMa = 0;
  float yMa = 0;
  float zMa = 0;

//   if (ge->rot[1] != 0) {
//     cout << "rangerDomWin::accum: module " << ge 
// 	 << " is rotated!" << endl;
//   }

  xyzMinMax(xMi,xMa,yMi,yMa,zMi,zMa,ge);

  if (nShad) {
    if (xMi < xMin) xMin = xMi;
    if (xMa > xMax) xMax = xMa;
    if (yMi < yMin) yMin = yMi;
    if (yMa > yMax) yMax = yMa;
    if (zMi < zMin) zMin = zMi;
    if (zMa > zMax) zMax = zMa;
 
  }
  else {
    xMin = xMi;
    xMax = xMa;
    yMin = yMi;
    yMax = yMa;
    zMin = zMi;
    zMax = zMa;
  }
  shadows.push_back(ge);
  ++nShad;
}


void rangerDomWin::addEntrance(const int dir,rangerDomLay* dl) {
  switch (dir) {
  case 1:
    dl->updNxt(firstLay);
    entranceLay = dl;
    break;
  case -1:
    dl->updPrv(lastLay);
    exitLay = dl;
    break;
  default:
    cout << "rangerDomWin::addEntrance: invalid dir=" << dir << endl;
  }
}



void rangerDomWin::rotatePoint(float& x,float& y,float& z,
			       const ArtePointer<GEDE> ge,const int dir) {
  float xx = x;
  float yy = y;
  float zz = z;

  switch (dir) {

  case -1:
    // transform from Arte coordinate system to plane coordinate system
    // (see Hera-B 97-090, p.67f)
    x = xx * ge->rot[0] + yy * ge->rot[3] + zz * ge->rot[6];
    y = xx * ge->rot[1] + yy * ge->rot[4] + zz * ge->rot[7];
    z = xx * ge->rot[2] + yy * ge->rot[5] + zz * ge->rot[8];
    break;

  case +1:
    // transform from plane coordinate system to Arte coordinate system
    x = xx * ge->rot[0] + yy * ge->rot[1] + zz * ge->rot[2];
    y = xx * ge->rot[3] + yy * ge->rot[4] + zz * ge->rot[5];
    z = xx * ge->rot[6] + yy * ge->rot[7] + zz * ge->rot[8];
    break;

  default:
    cout << "rangerDomWin::rotatePoint: invalid dir=" << dir << endl;
  }

}


void rangerDomWin::xyzMinMax(float& xMi,float& xMa,
			     float& yMi,float& yMa,
			     float& zMi,float& zMa,
			     const ArtePointer<GEDE> ge){
  
  float x = ge->sxmn;
  float y = ge->symn;
  float z = ge->zmin;

  rotatePoint(x,y,z,ge,+1);

  xMi = x;
  xMa = x;
  yMi = y;
  yMa = y;
  zMi = z;
  zMa = z;

  x = ge->sxmx;
  y = ge->symn; 
  z = ge->zmin;

  rotatePoint(x,y,z,ge,+1);

  if (x < xMi) xMi = x;
  if (x > xMa) xMa = x;
  if (y < yMi) yMi = y;
  if (y > yMa) yMa = y;
  if (z < zMi) zMi = z;
  if (z > zMa) zMa = z;

  x = ge->sxmn;
  y = ge->symx; 
  z = ge->zmin;

  rotatePoint(x,y,z,ge,+1);

  if (x < xMi) xMi = x;
  if (x > xMa) xMa = x;
  if (y < yMi) yMi = y;
  if (y > yMa) yMa = y;
  if (z < zMi) zMi = z;
  if (z > zMa) zMa = z;

  x = ge->sxmx;
  y = ge->symx; 
  z = ge->zmin;

  rotatePoint(x,y,z,ge,+1);

  if (x < xMi) xMi = x;
  if (x > xMa) xMa = x;
  if (y < yMi) yMi = y;
  if (y > yMa) yMa = y;
  if (z < zMi) zMi = z;
  if (z > zMa) zMa = z;

  x = ge->sxmn;
  y = ge->symn; 
  z = ge->zmax;

  rotatePoint(x,y,z,ge,+1);

  if (x < xMi) xMi = x;
  if (x > xMa) xMa = x;
  if (y < yMi) yMi = y;
  if (y > yMa) yMa = y;
  if (z < zMi) zMi = z;
  if (z > zMa) zMa = z;

  x = ge->sxmx;
  y = ge->symn; 
  z = ge->zmax;

  rotatePoint(x,y,z,ge,+1);

  if (x < xMi) xMi = x;
  if (x > xMa) xMa = x;
  if (y < yMi) yMi = y;
  if (y > yMa) yMa = y;
  if (z < zMi) zMi = z;
  if (z > zMa) zMa = z;

  x = ge->sxmn;
  y = ge->symx; 
  z = ge->zmax;

  rotatePoint(x,y,z,ge,+1);

  if (x < xMi) xMi = x;
  if (x > xMa) xMa = x;
  if (y < yMi) yMi = y;
  if (y > yMa) yMa = y;
  if (z < zMi) zMi = z;
  if (z > zMa) zMa = z;

  x = ge->sxmx;
  y = ge->symx; 
  z = ge->zmax;

  rotatePoint(x,y,z,ge,+1);

  if (x < xMi) xMi = x;
  if (x > xMa) xMa = x;
  if (y < yMi) yMi = y;
  if (y > yMa) yMa = y;
  if (z < zMi) zMi = z;
  if (z > zMa) zMa = z;

}


void rangerDomWin::updLsn() {
  // set lsn
  maxLsn[0] = 0;
  maxLsn[1] = 0;
  maxLsn[2] = 0;
  for (vector<rangerDomLay*>::iterator dli=DomLays.begin();
       dli != DomLays.end(); ++dli) {
    (*dli)->updLsn(maxLsn);
  }
  
  // set prv
  rangerDomLay* prv[3];
  for (int m=0; m<=2; ++m) {
    prv[m] = 0;
  }
  for (vector<rangerDomLay*>::iterator dli=DomLays.begin();
       dli != DomLays.end(); ++dli) {
    (*dli)->updPrv(prv);
    prv[0] = *dli;
    if ((*dli)->meth() == 1) {
      prv[1] = *dli;
    }
    else if ((*dli)->meth() == 2) {
      prv[2] = *dli;
    }
    // set firstLay
    if (! firstLay[0]) firstLay[0] = *dli;
    if ((*dli)->meth() == 1 || (*dli)->meth() == 2) {
      if (! firstLay[(*dli)->meth()]) firstLay[(*dli)->meth()] = *dli;
    }
  }

  // set nxt
  rangerDomLay* nxt[3];
  for (int m=0; m<=2; ++m) {
    nxt[m] = 0;
  }
  for (vector<rangerDomLay*>::reverse_iterator dli=DomLays.rbegin();
       dli != DomLays.rend(); ++dli) {
    (*dli)->updNxt(nxt);
    nxt[0] = *dli;
    if ((*dli)->meth() == 1) {
      nxt[1] = *dli;
    }
    else if ((*dli)->meth() == 2) {
      nxt[2] = *dli;
    }
    // set lastLay
    if (! lastLay[0]) lastLay[0] = *dli;
    if ((*dli)->meth() == 1 || (*dli)->meth() == 2) {
      if (! lastLay[(*dli)->meth()]) lastLay[(*dli)->meth()] = *dli;
    }
  }
}


int rangerDomWin::setProxyLayers() {
  int nLayersSet = 0;
  for (vector<rangerDomLay*>::iterator dli=DomLays.begin();
       dli != DomLays.end(); ++dli) {
    int l = (*dli)->getLsn(0);  // layer sequence number for method 0
    for (vector<rangerProxyWin*>::iterator 
	   pwi=(*dli)->win()->proxywins().begin();
	 pwi != (*dli)->win()->proxywins().end(); ++pwi) {
      // find layer number of same lsn in proxy window
      rangerDomLay* dl = (*pwi)->toWin()->getDlByLsn(l,0);
      if (dl) {
	if (fabs((*dli)->alpha() - dl->alpha()) > 0.01) 
	  cout << "rangerDomWin::setProxyLayers(): stereo angles incompatible "
	       << " window " << *(*dli)->win()
	       << " proxy layer " << *dl << endl;
	(*dli)->addProxyLay(new rangerProxyLay(*pwi,dl));
	++nLayersSet;
      }
    }
  }
  return nLayersSet;
}


rangerDomLay* rangerDomWin::getDlByLsn(const int lsn,const int method){
  for (rangerDomLay* dl=firstLay[method]; dl != 0; dl = dl->next(0,1)) {
    if (dl->getLsn(0) == lsn) return dl;
  }
  return 0;  // if nothing found
}



ostream& operator<<(ostream &os,const rangerDomWin &w){
  os << "(" << w.dom()->nam() << "|" << w.cmp() << "|" << w.sec() << ")";
  return os;
}


void rangerDomWin::dump(const int level) {
  if (level > 0) {
    cout << "  DomWin " << dom()->nam() << " cmp=" << Cmp << " sec=" << Sec
	 << " nShad=" << nShad 
	 << " xMin=" << xMin << " xMax=" << xMax
	 << endl << "                            "
	 <<  "     yMin=" << yMin << " yMax=" << yMax
	 << endl << "                            "
	 <<  "     zMin=" << zMin << " zMax=" << zMax << endl;
    cout << "                                  maxLsn="
	 << maxLsn[0] << " " << maxLsn[1] << " " << maxLsn[2] << endl;
    cout << "                                 firstDL=";
    if (firstLay[0]) cout << *firstLay[0];
    cout << "/";
    if (firstLay[1]) cout << *firstLay[1];
    cout << "/";
    if (firstLay[2]) cout << *firstLay[2];
    cout << " lastDL= ";
    if (lastLay[0]) cout << *lastLay[0];
    cout << "/";
    if (lastLay[1]) cout << *lastLay[1];
    cout << "/";
    if (lastLay[2]) cout << *lastLay[2];
    cout << endl;
    for (vector<rangerDomLay*>::iterator dli=DomLays.begin();
	 dli != DomLays.end(); ++dli) {
      (*dli)->dump(level-1);
    }
    if (level > 1) {
      cout << "  Entrance/exit:" << endl;
      if (entranceLay) entranceLay->dump(level-1);
      if (exitLay) exitLay->dump(level-1);

    }
  }
}
