// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
//  Modifications:
// - Introduced ::chiSqProb             14-Aug-00 Wouter Hulsbergen
                         
#include <algo.h>
#include "rang/RMap.hh"
#include "rang/Node.hh"
//#include "rang/mmbook.hh"
#include "arte/ArteMessage.hh"

rangerRMap::rangerRMap(ArtePointer<RSEG> rseg){
  rt = 0;
  rs = rseg;

  if (rseg != 0) {
    copy( rseg->rpnt.begin(), rseg->rpnt.end(), back_inserter( RPList ) );  
  }
}

rangerRMap::rangerRMap(ArtePointer<RTRA> rtra){
  rt = rtra; 
  rs = 0;

  if (rtra != 0) {
    copy( rtra->rpnt.begin(), rtra->rpnt.end(), back_inserter( RPList ) );  
  }
}

rangerRMap::rangerRMap(const rangerRMap& rm) 
{
  rt = rm.rt;
  rs = rm.rs;
  //cout << "...begin to copy the HMList" << endl;

  for (list<rangerHitMap>::const_iterator hmIt=(rm.HMList).begin();
       hmIt != (rm.HMList).end(); hmIt++) {
//     cout << "copy hit with Rhit=" << (*hmIt).getRhit() << endl;
    HMList.push_back(*hmIt);
  }


//   copy((rm.HMList).begin(),(rm.HMList).end(),HMList.begin());
//   cout << "                       done" << endl;
  //cout << "RMap copy constructor: copy the RPList" << endl;

  copy((rm.RPList).begin(),(rm.RPList).end(),back_inserter(RPList));

}


rangerRMap& rangerRMap::operator=(const rangerRMap& rm){
  cout << "rangerMap assignment operator!" << endl;
  rt = rm.rt;
  rs = rm.rs;
  HMList.erase(HMList.begin(),HMList.end());
  copy((rm.HMList).begin(),(rm.HMList).end(),HMList.begin());
  RPList.erase(RPList.begin(),RPList.end());
  copy((rm.RPList).begin(),(rm.RPList).end(),RPList.begin());
  // cout << "Old Hitmap size=" << (rm.HMList).size()
  //      << " new one " << HMList.size() << endl;
  return *this;
}




ostream& operator<<(ostream &os,const rangerRMap &rmap) {
  os << " rseg=" << rmap.rs << " rtra=" << rmap.rt
     << " #HM=" << (rmap.HMList).size();
  return os;
}

const ArtePointer<RHIT> rangerRMap::getRhit(rangerNode* np) const {
  rangerHitMap hm0(np);
  list<rangerHitMap>::const_iterator HMIt = 
    find(HMList.begin(),HMList.end(),hm0);
  if (HMIt != HMList.end()) {
    return (*HMIt).getRhit();
  }
  else {
    return 0;
  }
}

const ArtePointer<RPNT> rangerRMap::getRpnt(rangerNode* np) const {
  rangerHitMap hm0(np);
  list<rangerHitMap>::const_iterator HMIt = 
    find(HMList.begin(),HMList.end(),hm0);
  if (HMIt != HMList.end()) {
    return (*HMIt).getRpnt();
  }
  else {
    return 0;
  }
}

void rangerRMap::addHitMap(const rangerHitMap& hm){
  HMList.push_back( hm );
}


void rangerRMap::prepareIdPnt(){
  static int init=0;
  static float zDef[4]= { 854, 1174, 1214, 1290 }; // default positions
  static float zPnt[4]= {0, 0, 0, 0};
  static float zTol=50;

  if (! init) {
    zPnt[0] = 0;
    zPnt[1] = 0;
    zPnt[2] = 0;
    zPnt[3] = 0;

    for (ArteTable<GESL>::iterator geIt = ArteTable<GESL>::begin();
	 geIt != ArteTable<GESL>::end(); ++geIt) {
      if (geIt->cmp == 5) {     // RICH superlayer
	if (zPnt[0] == 0 || geIt->z1 < zPnt[0]) zPnt[0] = geIt->z1;
	if (zPnt[1] == 0 || geIt->z2 > zPnt[0]) zPnt[1] = geIt->z2;
      }
      else {
	if (geIt->cmp == 6) {     // TRD superlayer
	  if (zPnt[2] == 0 || geIt->z1 < zPnt[2]) zPnt[2] = geIt->z1;
	  if (zPnt[3] == 0 || geIt->z2 > zPnt[3]) zPnt[3] = geIt->z2;
	}
      }
    }

    static int id;                    // check
    for (int i=0; i<4; i++){
      if (fabs(zPnt[i]-zDef[i]) > 100) {
	arteMsg << "RMap::preparePnt: zPnt[" << i 
		<< "] =" << zPnt[i]
		<< " fixed to " << zDef[i] << endl;
	arteMsg.warn(id);
	zPnt[i] = zDef[i];
      }
    }
    init = 1;
  }  // end initialization

  // loop over nodes in hit map (increasing z)
  rangerNode* nPnt[4] = {0, 0, 0, 0};
  for (list<rangerHitMap>::iterator HMIt=HMList.begin();
       HMIt != HMList.end();
       HMIt++) {
    rangerNode* n = (*HMIt).getNode();
    int ipnt;
    if (n->hasHitb()) {
      ipnt = 0;
      if (n->zNode() <= zPnt[ipnt] && n->zNode() > zPnt[ipnt] - zTol) {
	if (nPnt[ipnt]) {
	  if (nPnt[ipnt]->zNode() < n->zNode()) {
	    nPnt[ipnt] = n;
	  }
	}
	else {
	  nPnt[ipnt] = n;
	}
      }

      ipnt = 1;
      if (n->zNode() >= zPnt[ipnt] && n->zNode() < zPnt[ipnt] + zTol) {
	if (nPnt[ipnt]) {
	  if (nPnt[ipnt]->zNode() > n->zNode()) {
	    nPnt[ipnt] = n;
	  }
	}
	else {
	  nPnt[ipnt] = n;
	}
      }

      ipnt = 2;
      if (n->zNode() <= zPnt[ipnt] && n->zNode() > zPnt[ipnt] - zTol) {
	if (nPnt[ipnt]) {
	  if (nPnt[ipnt]->zNode() < n->zNode()) {
	    nPnt[ipnt] = n;
	  }
	}
	else {
	  nPnt[ipnt] = n;
	}
      }

      ipnt = 3;
      if (n->zNode() >= zPnt[ipnt] && n->zNode() < zPnt[ipnt] + zTol) {
	if (nPnt[ipnt]) {
	  if (nPnt[ipnt]->zNode() > n->zNode()) {
	    nPnt[ipnt] = n;
	  }
	}
	else {
	  nPnt[ipnt] = n;
	}
      }
    }
  }

  // weedout the duplicates
  for (int i=0; i<=3; i++) {
    for (int j=(i+1); j<=3; j++) {
      if (nPnt[j] == nPnt[i]) nPnt[j] = 0;
    }
  }

  // fill hit map entries
  for (int i=0; i<=3; i++) {
    if (nPnt[i]) {
//       list<rangerHitMap>::const_iterator HMIt = 
// 	find(HMList.begin(),HMList.end(),rangerHitMap hm0(nPnt[i]));
      rangerHitMap hm0(nPnt[i]);
      list<rangerHitMap>::iterator HMIt = 
	find(HMList.begin(),HMList.end(),hm0);
      if (HMIt != HMList.end()) {
	// no action if hit map entry has already associated RPNT
	if ((*HMIt).getRpnt() == 0) {
	  // check if RPNTs available in list
	  if (! RPList.empty()) {
	    // assign the existing RPNT
	    (*HMIt).setRpnt( *(RPList.begin()) );
	    // remove from RPNT list
	    RPList.pop_front();
	  }
	  else {
	    // create new row in RPNT
	    mmbook_(ArtePointer<RPNT>::getLink().getRef(),1);
	    ArtePointer<RPNT> rpnew( ArtePointer<RPNT>::nrow() );
	    (*HMIt).setRpnt( rpnew );
	  }
	}
      }
    }
  }

//   cout << "rangerRMap::preparePnt: the points are:" << endl;
//   for (int i=0; i<4; i++) {
//     cout << " i=" << i << " z=" << zPnt[i] << endl;
//     if (nPnt[i]) {
//       nPnt[i]->dump();
//     }
//     else {
//       cout << "n=0" << endl;
//     }
//   }
}



void rangerRMap::addPnt(rangerNode* np){
  //cout << "rangerRMap::addPnt... ";
  rangerHitMap hm0(np);   // find hit map entry for node
  list<rangerHitMap>::iterator HMIt = 
    find(HMList.begin(),HMList.end(),hm0);
  if (HMIt != HMList.end()) {
    //cout << "found...";
    // no action if hit map entry has already associated RPNT
    if ((*HMIt).getRpnt() == 0) {
      //cout << "vacant...";
      // check if RPNTs available in list
      if (! RPList.empty()) {
	//cout << "reuse RPList...";
	// assign the existing RPNT
	(*HMIt).setRpnt( *(RPList.begin()) );
	// remove from RPNT list
	RPList.pop_front();
      }
      else {
	// create new row in RPNT
	//cout << "create new RPNT...";
	mmbook_(ArtePointer<RPNT>::getLink().getRef(),1);
	ArtePointer<RPNT> rpnew( ArtePointer<RPNT>::nrow() );
	(*HMIt).setRpnt( rpnew );
	//cout << rpnew;
      }
    }
  }
  //cout << endl;
}


rangerNode* rangerRMap::getNodeZLow() const { 
  rangerNode* n = 0;
  for (list<rangerHitMap>::const_iterator HMIt=HMList.begin();
       HMIt != HMList.end();
       HMIt++) {
    if (n) {
      if (((*HMIt).getNode())->zNode() < n->zNode()) n = (*HMIt).getNode();
    }
    else {
      n = (*HMIt).getNode();
    }
  }
  return n;
}

rangerNode* rangerRMap::getNodeZHigh() const { 
  rangerNode* n = 0;
  for (list<rangerHitMap>::const_iterator HMIt=HMList.begin();
       HMIt != HMList.end();
       HMIt++) {
    if (n) {
      if (((*HMIt).getNode())->zNode() > n->zNode()) n = (*HMIt).getNode();
    }
    else {
      n = (*HMIt).getNode();
    }
  }
  return n;
}


double rangerRMap::chiSq() const {
  double sum = 0;
  for (list<rangerHitMap>::const_iterator HMIt=HMList.begin();
       HMIt != HMList.end();
       HMIt++) {
    if ((*HMIt).getNode()) sum += ((*HMIt).getNode())->getChiSq();
  }
  return sum;
}

extern "C" float prob_(const float&, const int&) ;
float rangerRMap::chiSqProb() const {
  float prob=-1;
  if( ! HMList.empty() ) {
    double chSq = 0;
    int ndof = 0;
    bool first=true;
    for (list<rangerHitMap>::const_iterator HMIt=HMList.begin();
	 HMIt != HMList.end();
	 HMIt++) {
      if ((*HMIt).getNode()) {
	chSq += ((*HMIt).getNode())->getChiSq();
	if(first) {
	  first=false;
	  ndof = -((*HMIt).getNode())->getState().dim() ;
	}
	ndof += ((*HMIt).getNode())->dim() ;
      }
    }
    if(ndof>0) prob = prob_(chSq,ndof) ;
  }
  return prob;
}
       

void rangerRMap::dump() const {
  cout << "rangerRMap::dump rt=" << rt << " rs=" << rs 
       << " Number of Rpnts=" << RPList.size() << " Listing:";

  for (list<ArtePointer<RPNT> >::const_iterator lr=RPList.begin();
       lr != RPList.end(); 
       lr++) {
    cout << " " << (*lr);
  } 
  cout << endl;
  cout << " Number of Hits=" << HMList.size() << endl;
  for (list<rangerHitMap>::const_iterator HMIt=HMList.begin();
       HMIt != HMList.end();
       HMIt++) {
    (*HMIt).dump();
  }
}

