// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modified:   
// - moved many routines to State.C
//                                     14-Aug-00 Wouter Hulsbergen

#include <iostream>
#include <math.h>
#include "rang/StateLI.hh"
#include "rang/zfpar.hh"
#include "rang/zfconst.hh"

//rangerStateLI::rangerStateLI() : rangerState(5) {} ;

int rangerStateLI::predict(const double& zDestination,
			 const double& ,rangerState*& s,
			 HepMatrix*& F) const {

  int newState     = (s == 0);  // denote if memory allocations exist

#ifdef FULL
  cout << "rangerStateLI::predict: zDestination=" << zDestination << endl;

  cout << "rangerStateLI::predict: targetState=" << s
       << " newState= " <<  newState << endl;
#endif

  if (newState) {
    s = new rangerStateLI();
    if (!s) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }
  if (!F) {
    F = new HepMatrix(dim(),dim());
    if (!F) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }

#ifdef FULL
  cout << "rangerStateLI::predict: after allocation: s->vec()=" << s->vec() 
       << endl << "       *F=" << *F << endl;
#endif

  //double deltaZ = fabs(zDestination - zState);
  s->setZ( zDestination );      // linear transport
  s->setX( x() + (zDestination - zState) * tx() );
  s->setY( y() + (zDestination - zState) * ty() );
  s->setTx( tx() );
  s->setTy( ty() );
  s->setI( I() );   // assume that TOF is not yet implemented
  s->setQpEst( qPEst() );

  for (int i=1; i<=dim(); i++) {
    for (int j=1; j<=dim(); j++) {
      (*F)(i,j) = 0.;
    }
  }
  (*F)(1,1) = 1;
  (*F)(2,2) = 1;
  (*F)(3,3) = 1;
  (*F)(4,4) = 1;
  (*F)(5,5) = 1;
  (*F)(1,3) = zDestination - zState;
  (*F)(2,4) = zDestination - zState;

#ifdef FULL
  cout << "rangerStateLI::predict: after calculation, *F=" << *F << endl;

  cout << "rangerStateLI::predict: the transported vector is"
       << s->vec() << endl;
#endif

  s->cov() = c.similarity(*F);

#ifdef FULL
  cout << "rangerStateLI::predict: the transported cov is"
       << s->cov() << endl;
#endif

  return 0;
}

