// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified: 
//           Wouter Hulsbergen                 Modified: 14-Aug-00
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerGeom
// static class with derived geometry information

#ifndef rangerGeom_HH
#define rangerGeom_HH

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#ifndef rangerVolume_HH
#include "rang/Volume.hh"
#endif

class rangerGeom {

public:
  static void update();           // initialize or update geometry table
  static void condUpdate();       // initialize or update geometry table
                                  //  only if number of planes has changed 
  static void reset();            // clear geometry table
  static void build();            // build geometry table
  static void gsort();            // sort geometry table

  static inline rangerVolume* vol
  (ArtePointer<GEDE> gede){ return *vGede[gede.index()-1];}  // lookup 
  static inline rangerVolume* vol
  (ArtePointer<GWAL> gwal){ return *vGwal[gwal.index()-1];}  // lookup 
  static inline vector<rangerVolume*>::iterator volN(int plane)
    { return ((plane>0)? vGede[plane-1] : vGwal[-plane-1]); }
  static inline int vsn(vector<rangerVolume*>::iterator v)
  { return (v-volumes.begin()); } // volume sequence number
  
  // return pointer to first volume with lower z in active volume list
  static vector<rangerVolume*>::const_iterator firstUpstreamActiveVolN(int plane) {
    return planeToUpstrActive[plane-firstPlane] ; }

private:
  static vector<rangerVolume*> volumes; // vector of pointers to rangerVolumes
  static vector<vector<rangerVolume*>::iterator> vGede;
  static vector<vector<rangerVolume*>::iterator> vGwal;
  
  // vector of pointers to active volumes
  static vector<rangerVolume*> activeVolumes;
  static int firstPlane ;
  static vector<vector<rangerVolume*>::const_iterator> planeToUpstrActive;
};

#endif
