// 
//  Author:  Rainer Mankel                     Created:  07-Mar-99
//           Humboldt University Berlin          
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//  Modified:
//    MC06 is included in SL MC6                21/08/00 A. Spiridonov

// rgdef creates an initial rangerDomainObj(ect) and sets defaults (!) 
// for domain definition, which will later be used for domain building 
// by rgdom. Geometry needs not to exist yet, so rgdef can be called at 
// a very early stage.

// Between rgdef and rgdom, this list can still be changed by eg activating
// or deactivating domains via user commands.

#include "rang/DomObj.hh"
#include "rang/Domain.hh"

rangerDomObj* rangerDO = 0;   // global DO pointer

extern "C" {
  void rgdef_() {
    //cout << "rgdef: rangerDO=" << rangerDO << endl;
    if (rangerDO) return;

    // create standard ranger domain object
    rangerDO = new rangerDomObj();

    // create default domains. All domains are active by default.

    // scope 1 domains (vertex detector)
    rangerDO->appDef(new rangerDomain("SI1",1,1,"SI01"));
    rangerDO->appDef(new rangerDomain("SI2",1,1,"SI02"));
    rangerDO->appDef(new rangerDomain("SI3",1,1,"SI03"));
    rangerDO->appDef(new rangerDomain("SI4",1,1,"SI04"));
    rangerDO->appDef(new rangerDomain("SI5",1,1,"SI05"));
    rangerDO->appDef(new rangerDomain("SI6",1,1,"SI06"));
    rangerDO->appDef(new rangerDomain("SI7",1,1,"SI07"));
    // scope 2 domains (magnet tracker)
    rangerDO->appDef(new rangerDomain("MC1",2,1,"MC01","MS01","SI08"));
    rangerDO->appDef(new rangerDomain("MC2",2,1,"MC02"));
    rangerDO->appDef(new rangerDomain("MC3",2,1,"MC03","MS03"));
    rangerDO->appDef(new rangerDomain("MC4",2,1,"MC04"));
    rangerDO->appDef(new rangerDomain("MC5",2,1,"MC05","MS05"));
    //    rangerDO->appDef(new rangerDomain("MC6",2,1,"MC06"));
    rangerDO->appDef(new rangerDomain("MC6",2,1,"MS06","MC06"));
    rangerDO->appDef(new rangerDomain("MC8",2,1,"MC08"));
    // scope 3 domains (pattern tracker)
    rangerDO->appDef(new rangerDomain("PC1",3,1,"PC01","MS10"));
    rangerDO->appDef(new rangerDomain("PC2",3,1,"PC02","MS11"));
    rangerDO->appDef(new rangerDomain("PC3",3,1,"PC03","MS12"));
    //    rangerDO->appDef(new rangerDomain("PC3",3,1,"PC03"));
    rangerDO->appDef(new rangerDomain("PC4",3,1,"PC04","MS13"));
    // scope 4 domains (trigger tracker)
    rangerDO->appDef(new rangerDomain("TC1",4,1,"TC01","MS14"));
    rangerDO->appDef(new rangerDomain("TC2",4,1,"TC02","MS15"));

  }
}
   
