// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// function rgidealm
// locate hits with MC information in a given z interval and perform track fit,
// for a given charged particle if it exceeds a given
// number of hits and a minimal momentum. 
// callable from Fortran

#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "rang/ParticleType.hh"
#include "rang/Fit.hh"
#include "rang/seg2track.hh"
#include "rang/Geom.hh"
#include "rang/tfdigi.hh"
#include "rang/tpname.hh"
#include "rang/tusomh.hh"
#include "rang/tfcharg.hh"
#include "rang/tfmt2seg.hh"
#include "rang/tffhist.hh"
#include "rang/FLAGHist.hh"
#include "rang/rginit.hh"

extern "C" {
  void rgidealm_(const int& imc,
		const float& pMin, // minimal momentum to be considered
		const int& hitMin,  // minimal number of hits
		const float& zMin, // lower z bound
		const float& zMax, // upper z bound
		const int& elecCorr // electron bremsstrahlung corr flag
		){

    // test initialization
    rginit_();
    rangerGeom::condUpdate();

    // initialize /zmh/ if needed
    tusomh_();

    ArtePointer<MTRA> mtra(imc);

    //int imc = mtra.index();

    if (! tfcharg_(imc)) return;  // test if decent charged particle

    if ( (mtra->px * mtra->px + mtra->py * mtra->py
	  +  mtra->pz * mtra->pz) < (pMin * pMin) ) return;

    int rs = tfmt2seg_(imc,hitMin,zMin,zMax);
    // tfdigi_(imc,zmin,zmax);
#ifdef FULL
    cout << "rgidealm: mtra=" << imc << " rs=" << rs 
	 << "total number of RSEGs=" << ArtePointer<RSEG>::nrow() 
	 << endl;
#endif
    if (rs > 0 & ((int) ArtePointer<RSEG>::nrow()) >= rs ) {
      ArtePointer<RSEG> rseg(rs);

      rangerParticleType pType;
      switch (((ArtePointer<TOPP>) mtra->topp)->gean) {
      case 2:
      case 3:
	pType =  ELECTRON;
	break;
      case 5:
      case 6:
	pType =  MUON;
	break;
      case 8:
      case 9:
	pType =  PION;
	break;
      case 11:
      case 12:
	pType =  KAON;
	break;
      case 14:
      case 15:
	pType =  PROTON;
	break;
      default:
	pType =  MUON;
	break;
      }

      int fitStatus = rangerFit(rseg,pType);
      if (fitStatus >=0) {
#ifdef rangerHIST
	ArtePointer<RTRA> rtra = rangerSeg2track(rseg);
	if (rtra.index() > 0) {
	  int rt1 = rtra.index();
	  int rt2 = rtra.index();
	  int hitffMin = 0;
	  int hitfmMin = 0;
	  float zmin = 0;
	  float zmax = 1500;
	  tffhist_(imc,rt1,rt2,hitffMin,hitfmMin,zmin,zmax);
	}
#endif
      }
      else {
	cout << " rgidealm: fitStatus=" << fitStatus << endl;
      }
    }
  }
}

