/*--------------------------------------------------------------------------*
 *                                                                          
 *              bit mask for reconstruction modules                         
 *                                                                          
 *    This bit mask is used to fill the reconstruction error mask in        
 *    in the EVRC bank. The bit is set if the data structure was not        
 *    be filled without errors.
 *                                                                          
 *    bit             ARTE table            reconstruction modules          
 *    RECO_RSEG_VDS   RSEG, HITB            CATS, HOLMES                    
 *    RECO_RSEG_ITR   RSEG, HITB                                            
 *    RECO_RSEG_OTR   RSEG, HITB            RANGER
 *
 *    RECO_RCCL       RCCL, HITC            CARE                            
 *    RECO_RSEG_VDS   RSEG, HITB            CATS, HOLMES                    
 *                                                                          
 *    RECO_RTRA_CHRG  RTRA                  RECO (matching, track           
 *                                                definition, RANGER refit  
 *    RECO_RTRA_NEUT  RTRA                  RECO (photons)                  
 *                                                                          
 *    RECO_PID_ECAL   RTRA likelihoods                                      
 *    RECO_PID_MUON   RTRA likelihoods      MURECB                          
 *    RECO_PID_RICH   RTRA likelihoods      RISE, RITER, RIRE               
 *    RECO_PID_TRD    RTRA likelihoods      TRDevnt                         
 *                                                                          
 *    RECO_RVER_MAIN  RVER                  VT, ROVER                       
 *    RECO_RVER_K0S   RVER                  V0FIT                           
 *
 *    RECO_OTR_PCUT   EVRC                  rcModSel
 *    RECO_RICH_OCUT  EVRC                     rcModSel
 *
 * changes:
 * 07 Aug 2000 (AG) added flags for OTR p-cut and RICH occpancy cut
 * 16 Aug 2000 (AG) moved selection flags to rcModSel.hh
 *--------------------------------------------------------------------------*/

#define RECO_RSEG_VDS   0x00000001
#define RECO_RSEG_SVD   0x00000001
#define RECO_RSEG_ITR   0x00000002
#define RECO_RSEG_OTR   0x00000004
#define RECO_RCCL       0x00000008

#define RECO_RTRA_CHRG  0x00000010
#define RECO_RTRA_NEUT  0x00000020

#define RECO_PID_ECAL   0x00000040
#define RECO_PID_MUON   0x00000080
#define RECO_PID_RICH   0x00000100
#define RECO_PID_TRD    0x00000200

#define RECO_RVER_MAIN  0x00000400
#define RECO_RVER_K0S   0x00000800
