// ********************************************************************
//
//  Routine:      rcSetRTRArelations
//  Author:       T.Oest
//
//  Description:  set missing RTRA relations
//
//  Modification:
//  -- include level parameter (level = 1 only RHIT)    T.Oest 27/03/00 
//  -- set RHITs for all tracking chambers              T.Oest 26/05/00
//  -- adapted for correct hitb order in rseg           W.Hulsbergen
//
// ********************************************************************

#include <iostream>
#include <set>
#include <math.h>
#include <vector>

#include <arte/arte.hh>
#include <arte/RSEG_cmp.hh>


// ********************************************************************
//  function
// ********************************************************************
void rcSetRTRArelations( ArtePointer<RTRA> rtra, int level){
  
  //-------------------------------------------------------------------
  // copy vds/otr hitb from rseg and set link to rtra track 
  //-------------------------------------------------------------------

  for( ArteRelationIterator<HITB> hitb = rtra->hitb.begin() ;
       hitb != rtra->hitb.end() ; hitb++) {
    cout << "Hits already assigned to rtra. Skipping. " 
	 << rtra << endl;
    return ;
  }

  for( ArteRelationIterator<RSEG> rseg = rtra->rseg.begin();
       rseg != rtra->rseg.end(); ++rseg ) {
    
    if (((*rseg)->fit !=7) &&
	
	((*rseg)->cmp & Rsegc::vxd)  != 0 ||
	((*rseg)->cmp & Rsegc::wch) != 0  ||
	((*rseg)->cmp & Rsegc::msg) != 0  ||
	((*rseg)->cmp & Rsegc::muon) != 0)  {
      
      // the createRelation inverts the order. we do not want that.
      if(level == 2) {
	vector<ArtePointer<HITB> > hitbs ;
	for( ArteRelationIterator<HITB> hitb = (*rseg)->hitb.begin();
	     hitb != (*rseg)->hitb.end(); ++hitb ) 
	  hitbs.push_back(*hitb) ;
	for(vector<ArtePointer<HITB> >::reverse_iterator 
	      hitb = hitbs.rbegin() ; hitb != hitbs.rend() ; hitb++)
	  createRelation( rtra, *hitb, &RTRA::hitb );
      }
      
      // create new rhits
      for( ArteRelationIterator<RHIT> rhit = (*rseg)->rhit.begin();
	   rhit != (*rseg)->rhit.end(); ++rhit) {
	ArtePointer<RHIT> rhitnew = ArteTable<RHIT>::push_back(RHIT()) ;
	rhitnew->x  = (*rhit)->x;
	rhitnew->y  = (*rhit)->y;
	rhitnew->tx = (*rhit)->tx;
	rhitnew->ty = (*rhit)->ty;
	rhitnew->q  = (*rhit)->q;
	rhitnew->p  = rtra->pf ; // !!
	rhitnew->rtra = rtra ;
	createRelation( (*rhit)->hitb, rhitnew, &HITB::rhit );
      } 
    }
  }
  
  if (level == 2) {
    
    //---------------------------------------------------------------
    // set link to ECAL hitc
    //---------------------------------------------------------------
    
    for( ArteRelationIterator<RCCL> rccl = rtra->rccl.begin();
	 rccl != rtra->rccl.end(); ++rccl ) {  
      for( ArteRelationIterator<HITC> hitc = (*rccl)->hitc.begin();
	   hitc != (*rccl)->hitc.end(); ++hitc ) {
	ArtePointer<HITC> Phitc(hitc);
	  createRelation( rtra, Phitc, &RTRA::hitc );
      }
    }
  
    //---------------------------------------------------------------
    // set link to RICH hitb
    //---------------------------------------------------------------
    
    for( ArteRelationIterator<RSEG> rseg = rtra->rseg.begin();
	 rseg != rtra->rseg.end(); ++rseg ) {
      if ((*rseg)->fit !=7){
	if (((*rseg)->cmp & Rsegc::rich)  != 0 )  {
	  
	  ArtePointer<RSEG> rich = *rseg;
	  
	  for( ArteRelationIterator<HITB> hitb = rich->hitb.begin();
	       hitb != rich->hitb.end(); ++hitb ) {
	    ArtePointer<HITB> Phitb(hitb) ; 
	    createRelation( rtra, Phitb, &RTRA::hitb );
	  }
	}
      }
    }
    
  }
}


