
// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
//
// *****************************************************************************

#include "reco/rcXkinema.h"

// Thorsten's bremsstrahlung recovery
float rc99Bremsstrahlung(ArtePointer<RTRA> rtra);
float kinema::findBremsstrahlung(ArtePointer<RTRA> t) {
  return rc99Bremsstrahlung(t);
}


HepLorentzVector kinema::FourVector(ArtePointer<RTRA> t, float m) {
  float p=1/fabs(t->pf);
  float pz=p/sqrt(t->txf*t->txf + t->tyf*t->tyf + 1);
  float px=pz*t->txf;
  float py=pz*t->tyf;
  return HepLorentzVector(px,py,pz,-sqrt(p*p+m*m));
}


extern "C"  void rkmain_(const int    & icovar, 
              const double & zout, 
              const double & fstep, 
              const double * parameters, 
              const double * prec, 
              const double * p_track, 
              const double * derivatives, 
              const int    & ierror);

int kinema::swim(const double *source,double *dest,const double zout) {
  const int    icovar=2;
  double       fstep=50;
  double       prec[4];
  double       derivatives[5][5];
  int          ierror;

  prec[0] = 0.001/fabs(zout-source[0]);
  prec[1] = prec[0];
  prec[2] = 0.0001/fabs(zout-source[0]);
  prec[3] = prec[2];

  rkmain_(icovar,zout,fstep,source,prec,dest,&derivatives[0][0],ierror);

  return ierror;
}


ArtePointer<TOPP> kinema::findTopp(int lund) {
  ArtePointer<TOPP> topp;
  for (ArteTable<TOPP>::iterator itopp=ArteTable<TOPP>::begin();
       itopp!=ArteTable<TOPP>::end(); ++itopp) {
    if ((*itopp)->lund==lund) { topp=(*itopp); break; }
  }
  return topp;
}

