// *****************************************************************************
//
// source:    L3_Arte.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************
#include "L3_Arte.hh"
#include <iostream>
#include <vector>
#include <CLHEP/Vector/ThreeVector.h>

#include "auto/MTMT.h"
#include "auto/RTHI.h"
#include "arte/RSEG_cmp.hh"

#include "L3_SpacePoint.hh"
#include "L3_Track.hh"
#include "L3_Event.hh"

#include "esp.h"
#include "event.h"

void L3_MakeRhits(ArtePointer<RSEG> );
void L3_MakeSegTra(ArtePointer<RSEG>, ArtePointer<RTRA> );
void L3_MakeRpntSegTra(vector<ArtePointer<RPNT> > rpnts,
		       ArtePointer<RSEG> rseg, ArtePointer<RTRA> rtra);
void L3_MakeRsegHitb(ArtePointer<HITB> hitb, ArtePointer<RSEG> rseg);
void L3_MakeRtraHitb(ArtePointer<HITB> hitb, ArtePointer<RTRA> rtra);

//extern "C" {
//  void mmbrel_( int& link, const int& row1, const int& row2 );
//}

L3_Arte::L3_Arte() {
}

L3_Arte::L3_Arte(const L3_Arte& a) {
}

L3_Arte& L3_Arte::operator = (const L3_Arte& a) {
  return *this;
}

void L3_Arte::flush() {
}

void L3_Arte::fill(const L3_Event& Ev) {
  vector<L3_Track> TrackList;
  TrackList = Ev.silicon().Tracks();
  for (vector<L3_Track>::iterator TrackIt = TrackList.begin();
       TrackIt != TrackList.end(); ++TrackIt) {
    L3_Track Track(*TrackIt);
    ArtePointer<RSEG> rseg = track2rseg(Track);
    ArtePointer<RTRA> rtra = track2rtra(Track);
    L3_MakeSegTra(rseg, rtra);
    MakeRPNT(Track, rseg, rtra);
  }
}

ArtePointer<RSEG> 
L3_Arte::track2rseg(L3_Track& Track) {
// create new entry in RSEG. 
  mmbook_(ArtePointer<RSEG>::getLink().getRef(),1);
  ArtePointer<RSEG> rseg( ArtePointer<RSEG>::nrow() );

  vector<L3_SpacePoint> PunktList = Track.getPunktList();
  L3_SpacePoint FirstPoint = *(PunktList.begin());
  L3_SpacePoint LastPoint = *(PunktList.begin());
  for (vector<L3_SpacePoint>::iterator RPIt = PunktList.begin();
       RPIt != PunktList.end(); ++RPIt) {
    if ( RPIt->Punkt().z() < FirstPoint.Punkt().z() ) { 
      FirstPoint = *RPIt;
    }
    if ( RPIt->Punkt().z() > FirstPoint.Punkt().z() ) { 
      LastPoint = *RPIt;
    }
  }

  Track.kalTransport(FirstPoint.Punkt().z()); 
  rseg->zf = Track.refz();
  rseg->xf = Track.x();
  rseg->yf = Track.y();
  rseg->txf = Track.mx();
  rseg->tyf = Track.my();
  rseg->pf = 0.;
  rseg->cvf[0] = Track.Cov(0,0);
  rseg->cvf[1] = Track.Cov(0,1);
  rseg->cvf[2] = Track.Cov(1,1);
  rseg->cvf[3] = Track.Cov(0,2);
  rseg->cvf[4] = Track.Cov(1,2);
  rseg->cvf[5] = Track.Cov(2,2);
  rseg->cvf[6] = Track.Cov(0,3);
  rseg->cvf[7] = Track.Cov(1,3);
  rseg->cvf[8] = Track.Cov(2,3);
  rseg->cvf[9] = Track.Cov(3,3);
  for (int i=10; i<=14; i++) {
    rseg->cvf[i] = 0.;
  }
  rseg->time = 0.;
  rseg->dtim = 0;

  Track.kalTransport(LastPoint.Punkt().z()); 
  rseg->ze = Track.refz();
  rseg->xe = Track.x();
  rseg->ye = Track.y();
  rseg->txe = Track.mx();
  rseg->tye = Track.my();
  rseg->pe = 0.;
  rseg->cve[0] = Track.Cov(0,0);
  rseg->cve[1] = Track.Cov(0,1);
  rseg->cve[2] = Track.Cov(1,1);
  rseg->cve[3] = Track.Cov(0,2);
  rseg->cve[4] = Track.Cov(1,2);
  rseg->cve[5] = Track.Cov(2,2);
  rseg->cve[6] = Track.Cov(0,3);
  rseg->cve[7] = Track.Cov(1,3);
  rseg->cve[8] = Track.Cov(2,3);
  rseg->cve[9] = Track.Cov(3,3);
  for (int i=10; i<=14; i++) {
    rseg->cve[i] = 0.;
  }
  rseg->chi = Track.chisqr();
  rseg->fit = 0;
  rseg->cmp = Rsegc::vxd;

  vector<L3_Hitb> hitlist= Track.HList();
  for (vector<L3_Hitb>::iterator hLIt = hitlist.begin();
       hLIt != hitlist.end(); ++hLIt) {
    ArtePointer<HITB> hitb(hLIt->hit());
    L3_MakeRsegHitb(hitb, rseg);
  }
  L3_MakeRhits(rseg);
  return rseg;
}

ArtePointer<RTRA> 
L3_Arte::track2rtra(L3_Track& Track) {
// create new entry in RTRA. 
  mmbook_(ArtePointer<RTRA>::getLink().getRef(),1);
  ArtePointer<RTRA> rtra( ArtePointer<RTRA>::nrow() );

  vector<L3_SpacePoint> PunktList = Track.getPunktList();
  L3_SpacePoint FirstPoint = *(PunktList.begin());
  L3_SpacePoint LastPoint = *(PunktList.begin());
  for (vector<L3_SpacePoint>::iterator RPIt = PunktList.begin();
       RPIt != PunktList.end(); ++RPIt) {
    if ( RPIt->Punkt().z() < FirstPoint.Punkt().z() ) { 
      FirstPoint = *RPIt;
    }
    if ( RPIt->Punkt().z() > FirstPoint.Punkt().z() ) { 
      LastPoint = *RPIt;
    }
  }
  float z = FirstPoint.Punkt().z() ;
  Track.kalTransport(z); 
  rtra->zf = Track.refz();
  rtra->xf = Track.x();
  rtra->yf = Track.y();
  rtra->txf = Track.mx();
  rtra->tyf = Track.my();
  rtra->pf = 0.;
  rtra->cvf[0] = Track.Cov(0,0);
  rtra->cvf[1] = Track.Cov(0,1);
  rtra->cvf[2] = Track.Cov(1,1);
  rtra->cvf[3] = Track.Cov(0,2);
  rtra->cvf[4] = Track.Cov(1,2);
  rtra->cvf[5] = Track.Cov(2,2);
  rtra->cvf[6] = Track.Cov(0,3);
  rtra->cvf[7] = Track.Cov(1,3);
  rtra->cvf[8] = Track.Cov(2,3);
  rtra->cvf[9] = Track.Cov(3,3);
  for (int i=10; i<=14; i++) {
    rtra->cvf[i] = 0.;
  }

  rtra->hsi = Track.numHits();
  rtra->hin = 0;
  rtra->hou = 0;
  rtra->hri = 0;
  rtra->hca = 0;
  rtra->htr = 0;
  rtra->hmu = 0;
  rtra->fit = 0;
  rtra->chi = Track.chisqr();
  vector<L3_Hitb> hitlist= Track.HList();
  for (vector<L3_Hitb>::iterator hLIt = hitlist.begin();
       hLIt != hitlist.end(); ++hLIt) {
    ArtePointer<HITB> hitb(hLIt->hit());
    L3_MakeRtraHitb(hitb, rtra);
  }
  return rtra;
}

void L3_Arte::MakeRPNT(L3_Track& Track, ArtePointer<RSEG> rseg, ArtePointer<RTRA> rtra) {
  vector<ArtePointer<RPNT> > rpnts;
  vector<L3_SpacePoint> PunktList = Track.getPunktList();
  for (vector<L3_SpacePoint>::iterator RPIt = PunktList.begin();
       RPIt != PunktList.end(); ++RPIt) {
    mmbook_(ArtePointer<RPNT>::getLink().getRef(),1);
    ArtePointer<RPNT> rp( ArtePointer<RPNT>::nrow() );
    rp->zf = RPIt->Punkt().z();
    rp->xf = RPIt->Punkt().x();
    rp->yf = RPIt->Punkt().y();
    Track.kalTransport(RPIt->Punkt().z());
    rp->txf = Track.mx();
    rp->tyf = Track.my();
    rp->pf =  0.;
    rp->cvf[0] = Track.Cov(0,0);
    rp->cvf[1] = Track.Cov(0,1);
    rp->cvf[2] = Track.Cov(1,1);
    rp->cvf[3] = Track.Cov(0,2);
    rp->cvf[4] = Track.Cov(1,2);
    rp->cvf[5] = Track.Cov(2,2);
    rp->cvf[6] = Track.Cov(0,3);
    rp->cvf[7] = Track.Cov(1,3);
    rp->cvf[8] = Track.Cov(2,3);
    rp->cvf[9] = Track.Cov(3,3);
    for (int i=10; i<=14; i++) {
      rp->cvf[i] = 0.;
    }
    rpnts.push_back( rp );
  }
  L3_MakeRpntSegTra(rpnts, rseg, rtra);
}

char* L3_Arte::particleName(char *s)
{
  return Cstring(s,sizeof(TOPP::name));
}

char* L3_Arte::Cstring(char *s,int n){
  static char d[128];
  strncpy(d,s,n);
  d[n]=0;
  for(--n;n>=0 && d[n]==' ';n--) d[n]=0;
  return d;
}

void L3_Arte::hitb2svd( ArtePointer<HITB> i,int *sl,int *sec,int *lay){
  ArtePointer<GEDE> g(i->gede);      
  if( sec!=NULL ) *sec = g->sec;
  if( lay!=NULL ) *lay = g->lay;
  if( sl != NULL ){
    ArtePointer<GESL> ge = g->gesl;
    char buf[3];
    strncpy(buf,ge->name+2,2);
    buf[2]=0;
    *sl = atoi(buf);      
  }
}

int L3_Arte::svdHitsPresent(){
  if (ArteTable<HITB>::booked() && ArtePointer<HITB>::nrow()>0 ) {
    for (ArteTable<HITB>::iterator hitbIt=ArteTable<HITB>::begin(); 
	 hitbIt!=ArteTable<HITB>::end(); ++hitbIt ) {
      ArtePointer<HITB>& hitb = *hitbIt;
      ArtePointer<GEDE> Plane(hitb->gede);
      if( Plane->cmp == 1 ) return 1;
    }
  }
  return 0;
}

int L3_Arte::Btag(ArtePointer<MTRA> m,int depth){
  //for(int i=0;i<depth;i++) cout << ' ';
  //cout << L3_Arte::particleName(m->topp->name) << endl;
  if( m->topp->name[0]=='B' ) return depth;
  ArtePointer<MTRA> parent = m->btra;
  if( parent==ArtePointer<MTRA>(0) ) 
    return 0;
  else 
    return Btag(parent,depth+1);
}

int L3_Arte::comesFrom(ArtePointer<MTRA> m,char *s){
  ArtePointer<MTRA> parent = m->btra;
  if( parent==ArtePointer<MTRA>(0) ) return 0;
  if(!strncmp(parent->topp->name,s,strlen(s))) return 1;
  return comesFrom(parent,s);
}

ArtePointer<MTRA> L3_Arte::particleOrigin(ArtePointer<MTRA> m){
  ArtePointer<MTRA> parent = m->btra;
  if( parent==ArtePointer<MTRA>(0) ){
    return m;
  }
  return particleOrigin(parent);
}

int L3_Arte::fillDTLT(int *data,int length){  
  int off=1;
  int skipHeader = ArteTable<DTLT>::booked() && ArtePointer<DTLT>::nrow()>0;  
  // book rows
  if( skipHeader ){
    off += ArtePointer<DTLT>::nrow();
    mmbook_( ArtePointer<DTLT>::getLink().getRef(), length );
  } else {
    mmbook_( ArtePointer<DTLT>::getLink().getRef(), length+1 );
  }
  // check
  if( ArteTable<DTLT>::size() == 0 ){
    if(L3_Parameter::Debug()) cout << "ERROR: Creation of DTLT failed." << endl;
    return 0;
  }  
  // get Arte pointer to table to extract address
  int *dest = (int*)&ArtePointer<DTLT>(off)->bits;
  if( !skipHeader ){
    int recid = RID_NORMAL_SLB0;  /* Normal standard SLB    */
    // put recid in first row of 4 bytes (convention!)
    ESP_put_int( &recid,dest++, 1 );
  }  
  // copy data to following rows and up
  for(;length>0;length--) ESP_put_int(data++,dest++,1);
  return 1;
}
 
int L3_Arte::showDTLT(){
  if( !ArteTable<DTLT>::booked() ){
    cout << "DTLT: not booked" << endl;
    return 0;
  }
  cout << "DTLT: " << ArtePointer<DTLT>::nrow() << " rows" << endl;
  int n=0;
  for(ArtePointer<DTLT> i=ArteTable<DTLT>::begin();i!=ArteTable<DTLT>::end();i++,n++){
    int *p = (int*)i->bits;
    cout << "DTLT: " << n << ' ' << *p << endl;
  }
  return n;
}
 






