// *****************************************************************************
//
// source:    L3_Hitb.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************
#ifndef _L3_HITB_HH_
#define _L3_HITB_HH_
#include "L3_Defines.hh"
#include "L3_Geometry.hh"
#include <iostream>
#include <iomanip>
#include <cmath>

#ifdef L3_ARTE_FRAME
#include "arte/arte.hh"
typedef ArtePointer<GEDE> gedePointer;
#endif

#ifdef L3_SLP_FRAME
typedef int gedePointer;
#include "vdsHITB.h"
#endif

class L3_Hitb {
public:
  inline L3_Hitb();
  inline L3_Hitb(const L3_Hitb&);
#ifdef L3_ARTE_FRAME
  inline L3_Hitb(const ArtePointer<HITB>&);
#endif
#ifdef L3_SLP_FRAME
  inline L3_Hitb(const int& nr, const VDS_hitb&);
#endif
  inline L3_Hitb& operator = (const L3_Hitb&);

  inline bool operator < (const L3_Hitb&) const;  
  inline int operator == (const L3_Hitb&) const;
  inline int operator != (const L3_Hitb&) const;
  
  inline ostream& operator << (const L3_Hitb&);

  inline int   hit()  const { return  the_hit; } // hitb number
  inline float z()    const { return  the_z; } // z coordinate
  inline float x()    const { return  the_x; } // x coordinate
  //inline float y()    const { return  the_y; } // y coordinate (for pad chambers only)
  //inline float q()    const { return  the_q; } // Quantity: amplitude for SVD, MSGC
                      // drift time for outer tracker
                      // 0 for RICH, TRD, MUON
  //inline float r()    const { return  the_r; } // Resolution of quantity Q
  //inline gedePointer gede() const { return  the_gede; } // detector plane
  inline int   gesl() const { return  the_gesl; } // detector superlayer
  inline int   sec()  const { return  the_sec; } // detector sector
  inline int   lay()  const  { return  the_lay; } // detector layer
  inline int   chip() const { return (the_lay<3) ? 1 : 2; }
  inline void  setFlag(int f) { the_flag=f; }
  inline int   getFlag() const { return the_flag; }
  inline const L3_Plane& getPlane() const; //return corresponding L3_Plane
  inline L3_Plane plane() const; //return corresponding L3_Plane
  inline int isXHit() const;
  inline int isYHit() const;
private:
  int   the_hit; // hitb number
  float the_z;   // z coordinate
  float the_x;   // x coordinate
  //float the_y;   // y coordinate (for pad chambers only)
  //float the_q;   // Quantity: amplitude for SVD, MSGC
                 // drift time for outer tracker
                 // 0 for RICH, TRD, MUON
  //float the_r;   // Resolution of quantity Q
  //gedePointer the_gede;// detector plane
  int   the_gesl;// detector superlayer
  int   the_sec; // detector sector
  int   the_lay; // detector layer
  int   the_flag;// a flag to mark a hitb as "used"
};

#include "L3_Hitb.icc"
#endif
