#ifndef _L3_PLANE_HH_
#define _L3_PLANE_HH_
// *****************************************************************************
//
// source:    L3_Plane.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************

#include "L3_Defines.hh"
#include <iostream>
#include <iomanip>

#ifdef L3_ARTE_FRAME
#include "arte/arte.hh"
#endif
#ifdef L3_SLP_FRAME
#include "pltable.h"
#endif

class L3_Plane {
public:
  inline L3_Plane();
  inline L3_Plane(const L3_Plane&);
#ifdef L3_ARTE_FRAME
  inline L3_Plane(const ArtePointer<GEDE>&, const int&, const int&);
#endif
#ifdef L3_SLP_FRAME
  inline L3_Plane(PLANE*, const int&, const int&);
#endif
  inline L3_Plane(const float&, const int& nsl, const int& npl);

  inline L3_Plane& operator = (const L3_Plane&);

  inline int operator == (const L3_Plane&) const;
  inline int operator != (const L3_Plane&) const;

  inline ostream& operator << (const L3_Plane&);

  inline int   l3npl() const;   /* absolute number of plane               */
  inline int   l3nsl() const;   /* absolute number of superlayer          */
  inline int   cmp() const;     /* detector component 1 (SIL),2 (WCH),... */
  inline int   lay() const;     /* layer number within superlayer         */
  inline int   sec() const;     /* sector number                          */
//  inline int mod() const;     /* module number for outer tracker        */
  inline int   shp() const;     /* detector shape                         */
  inline float xmin() const;    /* x of the left  edge of the detector    */
  inline float xmax() const;    /* x of the right edge of the detector    */
  inline float ymin() const;    /* y of the left  edge of the detector    */ 
  inline float ymax() const;    /* y of the right edge of the detector    */
  inline float zmin() const;    /* z of the front side                    */
  inline float zmax() const;    /* z of the rear side                     */
  inline float sxmn() const;    /* x of the left  edge of the sens. area  */
  inline float sxmx() const;    /* x of the right edge of the sens. area  */
  inline float symn() const;    /* y of the left  edge of the sens. area  */
  inline float symx() const;    /* y of the right edge of the sens. area  */
  inline float cos() const;     /* cosine of the stereo angle             */
  inline float sin() const;     /* sine   of the stereo angle             */
  inline float icos() const;    /* (cosine  of the stereo angle)^-1       */
  inline float isin() const;    /* (sine    of the stereo angle)^-1       */
  inline float rot(int) const;  /* 3 by 3 rotation matrix                 */
  inline float acc() const;     /* measurement accuracy                   */
  inline float eff() const;     /* hit efficiency                         */
  inline float dod() const;     /* max drift distance or double hit reso  */
  inline float pit() const;     /* pitch                                  */
  inline float ms() const;      /* multiple scattering angle for 1 GeV    */
  inline float xfw(int) const;  /* x of first wire in sublayer            */
  inline float zfw(int) const;  /* z of first wire in sublayer            */
  inline int   nwi(int) const;  /* number of wires in sublayer            */
  inline int   nsl() const;     /* number of sublayers in segment         */

private:
  int   the_l3npl;   /* absolute number of plane               */ 
  int   the_l3nsl;   /* absolute number of superlayer          */ 
  int   the_cmp;     /* detector component 1 (SIL),2 (WCH),... */
  int   the_lay;     /* layer number within superlayer         */
  int   the_sec;     /* sector number                          */
//  int mod;     /* module number for outer tracker        */
  int   the_shp;     /* detector shape                         */
  float the_xmin;    /* x of the left  edge of the detector    */
  float the_xmax;    /* x of the right edge of the detector    */
  float the_ymin;    /* y of the left  edge of the detector    */ 
  float the_ymax;    /* y of the right edge of the detector    */
  float the_zmin;    /* z of the front side                    */
  float the_zmax;    /* z of the rear side                     */
  float the_sxmn;    /* x of the left  edge of the sens. area  */
  float the_sxmx;    /* x of the right edge of the sens. area  */
  float the_symn;    /* y of the left  edge of the sens. area  */
  float the_symx;    /* y of the right edge of the sens. area  */
  float the_cos;     /* cosine of the stereo angle             */
  float the_sin;     /* sine   of the stereo angle             */
  float the_icos;    /* (cosine  of the stereo angle)^-1       */
  float the_isin;    /* (sine    of the stereo angle)^-1       */
  float the_rot[9];  /* 3 by 3 rotation matrix                 */
  float the_acc;     /* measurement accuracy                   */
  float the_eff;     /* hit efficiency                         */
  float the_dod;     /* max drift distance or double hit reso  */
  float the_pit;     /* pitch                                  */
  float the_ms;      /* multiple scattering angle for 1 GeV    */
  float the_xfw[4];  /* x of first wire in sublayer            */
  float the_zfw[4];  /* z of first wire in sublayer            */
  int   the_nwi[4];  /* number of wires in sublayer            */
  int   the_nsl;     /* number of sublayers in segment         */
};

#include "L3_Plane.icc"

#endif
