// *****************************************************************************
//
// source:    L3_RFBand.hh
//
// type:      include file
//
// created:   05 Mar 2000
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************
#ifndef _L3_RFBAND_HH_
#define _L3_RFBAND_HH_
#include "L3_Defines.hh"
#include <iostream.h>
#include <vector>

#ifdef L3_ARTE_FRAME
#include "arte/arte.hh"
#endif

class L3_Track;

class L3_RFBand {
public:
  L3_RFBand(){}
  L3_RFBand(const L3_RFBand& a){ operator=(a); }
#ifdef L3_ARTE_FRAME
  L3_RFBand(const ArtePointer<GWAL>&);
#endif
  L3_RFBand& operator = (const L3_RFBand&);

  inline float dx() const { return  t_dx; } // extension in x 
  inline float dy() const { return  t_dy; } // extension in y
  inline float dz() const { return  t_dz; } // extension in y

  inline float xmin() const { return  t_xmin; } 
  inline float xmax() const { return  t_xmax; } 
  inline float ymin() const { return  t_ymin; } 
  inline float ymax() const { return  t_ymax; } 
  inline float zmin() const { return  t_zmin; } 
  inline float zmax() const { return  t_zmax; } 

  inline float slope() const { return  t_slope; }
  inline float zstop() const { return  t_zstop; }  
  float pathLength(const L3_Track &t) const; //return pathlength of track in band, 
  //lower than 0.0 if not hit
  vector<float> crossingPoint(const L3_Track& t) const;
private:
  float t_dx,t_dy,t_dz;
  float t_xmin,t_xmax;
  float t_ymin,t_ymax;
  float t_zmin,t_zmax;
  float t_slope,t_zstop;
};

ostream& operator<<(ostream&,const L3_RFBand&);

#endif
