#ifndef _L3_VTX_HH_
#define _L3_VTX_HH_
// *****************************************************************************
//
// source:    L3_Vtx.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************

#include <CLHEP/Vector/ThreeVector.h>
#include "L3_Track.hh"
#include "L3_VTtypes.hh"

class L3_Vtx {
public:

  friend class L3_Vertex;

  inline L3_Vtx();
  inline L3_Vtx(Hep3Vector &guess);
  inline L3_Vtx(Hep3Vector &position,PCovMatrix &cov);
  inline L3_Vtx(double x,double y,double z);
  inline L3_Vtx(double x,double y,double z,PCovMatrix &cov);
  inline L3_Vtx(int index);
  inline ~L3_Vtx();

  L3_Vtx& operator = (const L3_Vtx&);

  inline int operator == (const L3_Vtx&) const;
  inline int operator != (const L3_Vtx&) const;
  inline operator int() const;

  inline int add(const vector<L3_Track>&,int flag=0); //add a vector of tracks to vertex
  int add(const L3_Track&,int flag=0);          //add a track to the vertex
  void smooth(float chicut);                    
  int vtdis();

  inline Hep3Vector vertex() const;
  inline PCovMatrix Cov() const; //return covariance matrix
  inline double Cov(int i,int j) const; //return element of cov-matrix, 0 <= i,j <= 2
  inline float x() const;  // vertex coordinates. Are available after adding tracks and calling
  inline float y() const;  // L3_Vtx::fit() with return value 0
  inline float z() const;
  inline float dx() const; 
  inline float dy() const; 
  inline float dz() const;
  inline float trkchi2(int nthTrack) const;

  inline static int maxVertex();

  inline float chi2() const; 
  int fit(int use_momentum=0);               //perform fit, returns 0 on success
  inline static void NewEvent();            //clear vt common block at a new event or new vertex
  inline static void toVt(L3_Track&);
  inline void fromVt(L3_Track&);
  void show_track();
  void showChi2();

  inline int nTrack() const;
  inline int TrackNr(int i) const;
  inline vector<int> OutgoingTracks() const; 
  inline vector<L3_Track> OutgoingL3Tracks() const; 
  inline void associate(const L3_Track &track);
  inline int Nr() const;
  
private:
  int Status;        // status flag of vt-package, 0 means ok
  int nr;            // vertex number (for bookkeeping in vt-package)
  int kill;          // a bookkeeping flag
  vector<L3_Track> AssociatedTracks;

  static PCovMatrix guess_PCovMatrix;
  static Point guess_Point;

  void Vtx_init(Point&,PCovMatrix&);
  void show_track(int);
};

ostream& operator<<(ostream&,const L3_Vtx&);

#include "L3_Vtx.icc"

#endif
