#ifndef __VTMASSCONSTR
#define __VTMASSCONSTR

// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   12. Sept 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: class for a mass constraint
//
// changes:
// 12 Sept 2000 (TG) creation
//
// *****************************************************************************

#include <iosfwd>
#include <vector>
#include "VtVector.hh"

namespace VERTEX {
  class Relation;

  typedef std::vector<Relation*>                 Rel_v;
  typedef std::vector<Relation*>::iterator       Rel_it;
  typedef std::vector<Relation*>::const_iterator Rel_cit;

  /** Mass Constraint class - \textbf{under construction}*/
  //============================================================================
  // Class MassC
  //============================================================================
  class MassC {
  public:
    /** @name Constructors */
    ///
    MassC();
    ///
    MassC(double m);
    ///
    ~MassC();

    /** @name Access functions */
    /// add Track to mass constraint
    inline bool push_back(Relation& r) { return push_back(&r); }
    /// add Track to mass contraint
    bool push_back(Relation* r);
    /// get mass constraint
    inline double mass() const { return m_mass; }
    /// set mass constraint
    inline void mass(double m) { m_mass = m; }

  private:
    double           m_mass;  // mass for mass constraint
    Rel_v            m_rel;   // list of tracks participating im mass constr.
  };
} // end of namespace VERTEX
#endif
