#ifndef __VTSYMMATRIX
#define __VTSYMMATRIX
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   21. Aug 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: class for symmetric matrices
//
// changes:
// 21 Aug 2000 (TG) creation
// 04 Sep 2000 (TG) modified return type of dsinv()
//
// *****************************************************************************

#include "VtSqMatrix.hh"

extern "C" {
  void dsinv_(int *, double *, int *, int *);
  void dsfact_(int *, double *, int *, int *, double *, int *);
}

namespace MATRIX {
  class VtVector;

  /** Class for symmetric matrices
   */
  //============================================================================
  // Class VtSymMatrix: base class for covariance matrices (symm, pos. def.)
  //============================================================================
  class VtSymMatrix : public VtSqMatrix {
  public:
    /** @name Constructors */
    ///
    VtSymMatrix(const unsigned int dim) : VtSqMatrix(dim) {}
    /// initalize diagonal elements
    VtSymMatrix(const unsigned int dim, double diag);
    ///
    VtSymMatrix(const VtSqMatrix& rhs) : VtSqMatrix(rhs) {
      //      Assert();
    }
    ///
    VtSymMatrix(const VtMatrix& rhs) : VtSqMatrix(rhs) {
      //      Assert();
    }
    virtual ~VtSymMatrix();
   
    /** @name Matrix operations */
    /// calc inverse using momentum or not
    bool invert(const bool use_momentum);
    /// transform to inverse
    bool VtDsinv(int dim = 0);
    /// return inverse
    const VtSymMatrix dsinv(int dim = 0) const;
    /// compute determinant via CERNLIB dsfact()
    virtual double det() const;
    /// compute $v^t*A*v$
    double product(const VtVector& rhs, unsigned int dim = 0) const;
    /// compute $B^t*A*B$
    const VtSymMatrix product(const VtMatrix& rhs) const;
    /// copy a smaller matrix at a certain place
    virtual void place_at(const VtMatrix& rhs,
			  const unsigned int row,
			  const unsigned int col);

    const VtSymMatrix& operator=(const VtSymMatrix& rhs);
    const VtSymMatrix  operator+(const VtSymMatrix& rhs) const;
    const VtSqMatrix   operator+(const VtSqMatrix&  rhs) const;
    const VtMatrix     operator+(const VtMatrix&    rhs) const;
    const VtSymMatrix  operator-(const VtSymMatrix& rhs) const;
    const VtSymMatrix  operator-(void)                   const;
    const VtSqMatrix   operator-(const VtSqMatrix&  rhs) const;
    const VtMatrix     operator-(const VtMatrix&    rhs) const;
    const VtSqMatrix   operator*(const VtSymMatrix& rhs) const;
    const VtSqMatrix   operator*(const VtSqMatrix&  rhs) const;
    const VtMatrix     operator*(const VtMatrix&    rhs) const;
    const VtVector     operator*(const VtVector&    rhs) const;

  private:
    void Assert(void);
  };
  
  //============================================================================
  // operator<<
  //============================================================================
  inline std::ostream& operator<< ( std::ostream& os, const VtSymMatrix& t ) {
    os.setf(ios::scientific,ios::floatfield);
    t.print(os);
    return os;
  }

} // end of namespace MATRIX
#endif
