*CMZ :          20/05/96  12.58.34  by  Unknown
*-- Author :   T.Lohse
c
c------------------------------------------------------------
c
c       subroutine:   VTFILT(iv,momflg,flag)
c
c       input:    iv       =  vertex index      Int
c                 momflg   =  fit type          Int
c                      0   =  straight lines
c                      1   =  including momenta
c
c       output:   flag   Int  < 0 if fail
c
c       action:   filters the tracks currently on vertex iv
c
c------------------------------------------------------------
c
      subroutine  VTFILT ( iv, momflg, flag )

      implicit none

      integer iv, momflg
      integer flag

      integer ifail
      integer i, j, k, l, n, m, it, imax, jmax

#include "vt/vtdat.fi"

      double precision 
     1  AGB(5,3), GBWBG(5,5), WBG(3,5), AGA(3,3)
      double precision 
     1  pc(5), AGpc(3), Cx(3), pcAx(5), Cwork(3,3), Wwork(3,3)
      double precision 
     1  z0, ax, ay
      double precision 
     1  xk0(3), qk0(3)
      double precision 
     1  chi2f
      double precision 
     1  a33, a34, b13

      double precision 
     1  VTCHI2

      flag = -1
      if (iv.gt.nvtmax)            goto 999
      if ( .not. vtset(iv) )       goto 999

      if (momflg.eq.0) then
        imax = 4
        jmax = 2
      else
        imax = 5
        jmax = 3
      endif

      n = ntr(iv)

      fit(iv) = 0
      smooth(iv) = 0
      dist(iv) = 0

c----------> loop over tracks on vertex

      z0 = xvs(3,iv)
      b13 = -z0
      Bk13(iv) = b13

      do  k = 1, n

      l = k-1

      ax = qvs(1,iv,k)
      ay = qvs(2,iv,k)

c----->  filtered covariance matrices

      a33 = -ax
      a34 = -ay
      Ak33(iv,k) = a33
      Ak34(iv,k) = a34

      it = ktr(iv,k)

      Wwork(1,1) = G(1,1,it)+2.d0*b13*G(1,3,it)+b13**2*G(3,3,it)
      Wwork(2,2) = G(2,2,it)+2.d0*b13*G(2,4,it)+b13**2*G(4,4,it)
      Wwork(1,2) = G(1,2,it)+b13*G(2,3,it)
     1             +b13*G(1,4,it)+b13**2*G(3,4,it)
      Wwork(2,1) = Wwork(1,2)
      if (momflg.eq.0) then
        call DSINV(2,Wwork,3,ifail)
        if ( ifail .eq. -1 ) goto 999
        W(1,1,iv,k) = Wwork(1,1)
        W(2,2,iv,k) = Wwork(2,2)
        W(1,2,iv,k) = Wwork(1,2)
        W(2,1,iv,k) = Wwork(1,2)
      else
        Wwork(1,3) = G(1,5,it)+b13*G(3,5,it)
        Wwork(2,3) = G(2,5,it)+b13*G(4,5,it)
        Wwork(3,3) = G(5,5,it)
        Wwork(3,1) = Wwork(1,3)
        Wwork(3,2) = Wwork(2,3)
        call DSINV(3,Wwork,3,ifail)
        if ( ifail .eq. -1 ) goto 999
        W(1,1,iv,k) = Wwork(1,1)
        W(2,2,iv,k) = Wwork(2,2)
        W(1,2,iv,k) = Wwork(1,2)
        W(2,1,iv,k) = Wwork(2,1)
        W(3,3,iv,k) = Wwork(3,3)
        W(3,2,iv,k) = Wwork(3,2)
        W(3,1,iv,k) = Wwork(3,1)
        W(1,3,iv,k) = Wwork(1,3)
        W(2,3,iv,k) = Wwork(2,3)
      endif

      do  i = 1, imax
        AGB(i,1) = G(1,i,it)+b13*G(3,i,it)
        AGB(i,2) = G(2,i,it)+b13*G(4,i,it)
      enddo
      if (momflg.ne.0) then
        do  i = 1, 5
          AGB(i,3) = G(i,5,it)
        enddo
      endif
      do  i = 1, jmax
        do  j = 1, imax
          WBG(i,j) = 0.
          do  m = 1, jmax
            WBG(i,j) = WBG(i,j) + W(i,m,iv,k)*AGB(j,m)
          enddo
        enddo
      enddo
      do  i = 1, imax
        do  j = i, imax
          GBWBG(i,j) = 0.
          do  m = 1, jmax
            GBWBG(i,j) = GBWBG(i,j) + AGB(i,m)*WBG(m,j)
          enddo
          GBWBG(j,i) = GBWBG(i,j)
        enddo
      enddo
      do  i = 1, imax
        do  j = i, imax
          GB(i,j,iv,k) = G(i,j,it)-GBWBG(i,j)
          GB(j,i,iv,k) = GB(i,j,iv,k)
        enddo
      enddo

      AGA(1,1) = GB(3,3,iv,k)
      AGA(1,2) = GB(3,4,iv,k)
      AGA(1,3) = GB(3,3,iv,k)*a33+GB(3,4,iv,k)*a34
      AGA(2,1) = AGA(1,2)
      AGA(2,2) = GB(4,4,iv,k)
      AGA(2,3) = GB(3,4,iv,k)*a33+GB(4,4,iv,k)*a34
      AGA(3,1) = AGA(1,3)
      AGA(3,2) = AGA(2,3)
      AGA(3,3) = GB(3,3,iv,k)*a33**2+GB(4,4,iv,k)*a34**2
     1           + 2.d0*GB(3,4,iv,k)*a33*a34

      do  i = 1, 3
        do  j = i, 3
          CINV(i,j,iv,k) = CINV(i,j,iv,l) + AGA(i,j)
          CINV(j,i,iv,k) = CINV(i,j,iv,k)
          Cwork(i,j) = CINV(i,j,iv,k)
          Cwork(j,i) = Cwork(i,j)
        enddo
      enddo
      call DSINV(3,Cwork,3,ifail)
      if ( ifail .eq. -1 )         goto 999
      do  i = 1, 3
        do  j = i, 3
          C(i,j,iv,k) = Cwork(i,j)
          C(j,i,iv,k) = Cwork(i,j)
        enddo
      enddo

c------->   filtered state vector

      pc(1) = p(1,it)
      pc(2) = p(2,it)
      pc(3) = p(3,it) - b13*a33
      pc(4) = p(4,it) - b13*a34
      if (momflg.ne.0) pc(5) = p(5,it)

      do  i = 1, 2
        AGpc(i) = 0.
        do  j = 1, imax
          AGpc(i) = AGpc(i) + GB(j,i+2,iv,k)*pc(j)
        enddo
      enddo
      AGpc(3) = 0.
      do  j = 1, imax
        AGpc(3) = AGpc(3) + pc(j) *
     1      ( a33*GB(j,3,iv,k) + a34*GB(j,4,iv,k) )
      enddo

      do  i = 1, 3
        Cx(i) = AGpc(i)
        do  j = 1, 3
          Cx(i) = Cx(i) + CINV(i,j,iv,l)*xv(j,iv,l)
        enddo
      enddo
      do  i = 1, 3
        xv(i,iv,k) = 0.
        do  j = 1, 3
          xv(i,iv,k) = xv(i,iv,k) + C(i,j,iv,k)*Cx(j)
        enddo
      enddo

      pcAx(1) = pc(1)
      pcAx(2) = pc(2)
      pcAx(3) = pc(3) - xv(1,iv,k) - a33*xv(3,iv,k)
      pcAx(4) = pc(4) - xv(2,iv,k) - a34*xv(3,iv,k)
      if (momflg.ne.0) pcAx(5) = pc(5)

      do  i = 1, jmax
        qv(i,iv,k) = 0.
        do  j = 1, imax
          qv(i,iv,k) = qv(i,iv,k) + WBG(i,j)*pcAx(j)
        enddo
      enddo

c-------> filtered chi**2

      do  i = 1, 3
        xk0(i) = xv(i,iv,k)
      enddo
      do  j = 1, jmax
        qk0(j) = qv(j,iv,k)
      enddo

      chi2f = VTCHI2(iv,k,xk0,qk0,b13,a33,a34,momflg)

      chi2(iv,k) = chi2f
      if ( k .gt. 1 ) chi2(iv,k)=chi2(iv,k)+chi2(iv,k-1)

      enddo

      if (momflg.eq.0) then
        fit(iv) = 1
      else
        fit(iv) = 2
      endif
      smooth(iv) = 0
      dist(iv) = 0
      flag = 0

999   continue

      return
      end
